from uhf.reader.protocol.message import Message
from uhf.reader.protocol.enumg import EnumG


class MsgAppReset(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_Reset.value

    def pack(self):
        super().pack()

    def unPack(self):
        pass



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetWhiteListSwitch(Message):

    def __init__(self, onOrOff: int, filterArea: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetWhiteListSwitch.value
        self.onOrOff = onOrOff
        self.filterArea = filterArea

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.onOrOff)
        buffer.putInt(self.filterArea)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Set Fail."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetGpiTrigger(Message):

    def __init__(self, gpiPort: int, triggerStart: int, hexTriggerCommand: str, triggerOver: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetGpiTrigger.value
        self.gpiPort = gpiPort
        self.triggerStart = triggerStart
        # self.triggerCommand = None
        self.hexTriggerCommand = hexTriggerCommand
        self.triggerOver = triggerOver
        self.overDelayTime = kwargs.get("overDelayTime", None)
        self.levelUploadSwitch = kwargs.get("levelUploadSwitch", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.gpiPort)
        buffer.putInt(self.triggerStart)
        if self.hexTriggerCommand:
            to_bytes = hexToBytes(self.hexTriggerCommand)
            buffer.putShort(len(to_bytes))
            buffer.putBytes(to_bytes)
        else:
            buffer.putShort(0)

        buffer.putInt(self.triggerOver)
        if self.overDelayTime is not None:
            buffer.putInt(0x01)
            buffer.putShort(self.overDelayTime)
        if self.levelUploadSwitch is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.levelUploadSwitch)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter reader hardware is not supported .",
                      2: "Parameters are missing ."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class LogGpiOver(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppLogMid_gpiOver.value
        self.gpiPort = None
        self.gpiPortLevel = None
        self.systemTime = None
        self.readerSerialNumber = None
        self.readerName = None

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.gpiPort = buffer.readInt()
            self.gpiPortLevel = buffer.readInt()
            second = buffer.readLong()
            mic = divmod(buffer.readLong(), 1000000)
            self.systemTime = secondFormat(second + mic[0])

    def __str__(self) -> str:
        return str((self.gpiPort, self.gpiPortLevel, self.systemTime))


from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseInventoryGJb(Message):

    def __init__(self, antennaEnable: int, inventoryMode: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_InventoryGJb.value
        self.antennaEnable = antennaEnable
        self.inventoryMode = inventoryMode
        self.filter = kwargs.get("filter", None)  # type:ParamEpcFilter
        self.readTid = kwargs.get("readTid", None)  # type:ParamEpcReadTid
        self.readUserData = kwargs.get("readUserData", None)  # type:ParamEpcReadUserData
        self.hexPassword = kwargs.get("hexPassword", None)
        self.safeMark = kwargs.get("safeMark", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaEnable)
        buffer.putInt(self.inventoryMode)
        if self.filter is not None:
            buffer.putInt(0x01)
            filter_bytes = self.filter.toBytes()
            buffer.putShort(len(filter_bytes))
            buffer.putBytes(filter_bytes)
        if self.readTid is not None:
            buffer.putInt(0x02)
            buffer.putBytes(self.readTid.toBytes())
        if self.readUserData is not None:
            buffer.putInt(0x03)
            buffer.putBytes(self.readUserData.toBytes())
        if self.hexPassword is not None:
            buffer.putInt(0x05)
            buffer.putBytes(hexToBytes(self.hexPassword))
        if self.safeMark is not None:
            buffer.putInt(0x06)
            buffer.putInt(self.safeMark)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter error.",
                      2: "Filter parameter error.", 3: "TID parameter error.", 4: "User parameter error.",
                      5: "Other error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.communication.tcp_client import TcpClient
from uhf.reader.communication.tcp_server import TcpServer
from uhf.reader.communication.gclient import GClient
from uhf.reader.protocol import *


class GServer(object):

    def __init__(self):
        self.__tcpServer = None
        self.__dicClients = {}
        self.__callGClientConnected = None

    def openServer(self, port: int, address="0.0.0.0"):
        if self.__tcpServer:
            return False
        self.__tcpServer = TcpServer()
        self.__tcpServer.onRemoteConnected = self.__processConnect
        if self.__tcpServer.open(port, address):
            return True
        else:
            self.close()
            return False

    def __processConnect(self, tcpClient: TcpClient):
        if not tcpClient:
            return
        g_client = GClient()
        if g_client.openServer(tcpClient):
            if tcpClient.addr not in self.__dicClients:
                self.__dicClients[tcpClient.addr] = g_client
            else:
                self.__dicClients[tcpClient.addr].close()
                self.__dicClients[tcpClient.addr] = g_client
            if self.__callGClientConnected:
                info = MsgAppGetReaderInfo()
                if g_client.sendSynMsg(info) == 0:
                    g_client.serialNumber = info.readerSerialNumber
                    self.__callGClientConnected(info.readerSerialNumber, g_client)
                else:
                    g_client.close()

    def close(self):
        if self.__tcpServer:
            self.__tcpServer.close()
            self.__tcpServer = None
            # self.closeAllClient()

    def closeClient(self, readerName: ()):
        if len(readerName):
            if readerName in self.__dicClients:
                self.__dicClients[readerName].close()
                del self.__dicClients[readerName]

    def closeAllClient(self):
        for client in self.__dicClients.values():
            client.close()
        self.__dicClients.clear()

    @property
    def callGClientConnected(self):
        return self.callGClientConnected

    @callGClientConnected.setter
    def callGClientConnected(self, call):
        self.__callGClientConnected = call



from uhf.reader.protocol import *
from uhf.reader.utils import *


class ParamEpcFastId(Parameter):

    def __init__(self, fastId: int, tagFoucs: int):
        self.fastId = fastId
        self.tagFoucs = tagFoucs

    def bytesToClass(self, dataBytes):
        hex = bytesToHex(dataBytes)
        buffer = DynamicBuffer("0x" + hex)
        self.fastId = buffer.readInt()
        self.tagFoucs = buffer.readInt()
        return self

    def toBytes(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.fastId)
        buffer.putInt(self.tagFoucs)
        return buffer.tobytes()

    def __str__(self) -> str:
        return str((self.fastId, self.tagFoucs))



# import hid
#
#
# def getUsbHidPathList():
#     e = hid.enumerate(0x03eb, 0x2421)
#     result = []
#     for item in e:  # type:dict
#         path = str(item.get("path"))
#         if path.find("kbd") == -1 and path.find("KBD") == -1:
#             result.append(item.get("path"))
#     return result
#
#
# def getUsbHidPathDict():
#     """
#     获取usb-hid字典
#     :return: usb-hid dict
#     """
#     result = {}
#     e = hid.enumerate(0x03eb, 0x2421)
#     for index, item in enumerate(e):  # type:dict
#         path = str(item.get("path"))
#         if path.find("kbd") == -1 and path.find("KBD") == -1:
#             result["USB-HID-" + str(index)] = item.get("path")
#     return result
#
#
# #
# #
# # print(getUsbHidPathList())
# # # print(getUsbHidPathDict())



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseSuperRW(Message):

    def __init__(self, antennaEnable: int, instructType: int, start: int, extraCode: str, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_SuperRW.value
        self.antennaEnable = antennaEnable
        self.instructType = instructType
        self.start = start
        self.extraCode = extraCode
        self.hexData = kwargs.get("hexData", None)
        self.readData = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaEnable)
        buffer.putInt(self.instructType)
        buffer.putShort(self.start)
        buffer.putBytes(hexToBytes(self.extraCode))
        if self.hexData:
            buffer.putInt(0x01)
            to_bytes = hexToBytes(self.hexData)
            buffer.putBytes(to_bytes)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter error.", 3: "Other error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)
            if len(self.cData) > 1:
                readBuffer = DynamicBuffer("0x" + bytesToHex(self.cData))
                readBuffer.pos = 8
                if readBuffer.readInt() == 1:
                    self.readData = bytesToHex(readBuffer.readBytes(16))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseGetGbBaseBand(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_GetGbBaseBand.value
        self.speed_tc = None
        self.speed_trext = None
        self.speed_k = None
        self.speed_miller = None
        self.cin = None
        self.ccn = None
        self.session = None
        self.inventoryFlag = None

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.speed_tc = buffer.readBitLen(1)
            self.speed_trext = buffer.readBitLen(1)
            self.speed_k = buffer.readBitLen(4)
            self.speed_miller = buffer.readBitLen(2)
            self.cin = buffer.readBitLen(4)
            self.ccn = buffer.readBitLen(4)
            self.session = buffer.readInt()
            self.inventoryFlag = buffer.readInt()
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.speed_tc, self.speed_trext, self.speed_k, self.speed_miller, self.cin, self.ccn, self.session,
                    self.inventoryFlag))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetTcpMode(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetTcpMode.value
        self.tcpMode = None
        self.serverPort = None
        self.clientIp = None
        self.clientPort = None

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.tcpMode = buffer.readInt()
            self.serverPort = buffer.readShort()
            self.clientIp = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt()) + "." + str(
                buffer.readInt())
            self.clientPort = buffer.readShort()
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.tcpMode, self.serverPort, self.clientIp, self.clientPort))
        # return "{0},{1},{2},{3},{4},{5}".format(self.readerSerialNumber, self.powerOnTime, self.baseCompileTime,
        #                                     self.appVersions, self.systemVersions, self.appCompileTime)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestSerialNoSet(Message):

    def __init__(self, readerSerialNumber: str):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_SerialNoSet.value
        self.readerSerialNumber = readerSerialNumber

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.readerSerialNumber is not None:
            buffer.putShort(len(self.readerSerialNumber))
            buffer.putBytes(self.readerSerialNumber.encode())

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success.", 1: "Other error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseSetFrequency(Message):

    def __init__(self, automatically: int, *args, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_SetFrequency.value
        self.automatically = automatically
        self.listFreqCursor = args
        self.powerDownSave = kwargs.get("powerDownSave", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.automatically)
        if self.automatically == 0 and self.listFreqCursor:
            buffer.putInt(0x01)
            buffer.putShort(len(self.listFreqCursor))
            for value in self.listFreqCursor:
                buffer.putInt(value)
        if self.powerDownSave is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.powerDownSave)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "The channel number is not in the current frequency band.",
                      2: "Invalid frequency points.", 3: "Other error.", 4: "Save failure."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.utils.HexUtils import bytesToHex
from uhf.reader.utils.byteBuffer import *


def getPcLen(value: str):
    """
    :rtype: int
    """
    if value:
        t1, t2 = divmod(len(value), 4)
        if t2 == 0:
            return t1
        elif t2 > 0:
            return t1 + 1


def getPcValue(value: str):
    """
    计算epc的pc值
    :param value: 16进制写入内容
    :return: 16进制 pc值
    """
    pcLen = getPcLen(value)
    iPc = pcLen << 11
    buffer = DynamicBuffer()
    buffer.putLong(iPc)
    buffer.pos = 16
    return bytesToHex(buffer.readBytes(16))


def getEpcData(value: str):
    """
    写6C epc 快捷返回pc+内容
    :param value: 16进制写入内容
    :return:pc+内容
    """
    value_len = getPcLen(value) * 4
    pc_value = getPcValue(value)
    return pc_value + value.ljust(value_len, "0")


def getGbPcValue(value: str):
    """
    计算GB 编码区的pc值
    :param value: 16进制写入内容
    :return: 16进制 pc值
    """
    pcLen = getPcLen(value)
    iPc = pcLen << 8
    buffer = DynamicBuffer()
    buffer.putLong(iPc)
    buffer.pos = 16
    return bytesToHex(buffer.readBytes(16))


def getGbData(value: str):
    """
    写GB 编码区 快捷返回pc+内容
    :param value: 16写入进制内容
    :return: pc+内容
    """
    value_len = getPcLen(value) * 4
    pc_value = getGbPcValue(value)
    return pc_value + value.ljust(value_len, "0")


if __name__ == '__main__':

    print(divmod(632,64))
    for i in range(1):
        print(i)
    l = [1, 2, 3, 4, 5]
    l.extend([4, 5, 6, 7])
    print(l)




from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseSetAntennaHub(Message):

    def __init__(self, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_SetAntennaHub.value
        self.dicHub = kwargs

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.dicHub:
            for key, value in self.dicHub.items():
                if str(key).isdigit():
                    buffer.putInt(int(key))
                    buffer.putShort(value)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter reader is not supported.",
                      2: "Save failure."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *


class MsgAppGetCacheTagData(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetCacheTagData.value
        self.switch = None

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Have  data.", 1: "No data.", 2: "End of data return."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



import threading
import contextlib
import queue
import time


class ThreadPool(object):
    def __init__(self, max_num):
        self.StopEvent = 0  # 线程任务终止符，当线程从队列获取到StopEvent时，代表此线程可以销毁。可设置为任意与任务有区别的值。
        self.q = queue.Queue()
        self.max_num = max_num  # 最大线程数
        self.terminal = False  # 是否设置线程池强制终止
        self.created_list = []  # 已创建线程的线程列表
        self.free_list = []  # 空闲线程的线程列表
        self.Deamon = False  # 线程是否是后台线程

    def run(self, func, args, callback=None):
        """
        线程池执行一个任务
        :param func: 任务函数
        :param args: 任务函数所需参数
        :param callback:
        :return: 如果线程池已经终止，则返回True否则None
        """

        if len(self.free_list) == 0 and len(self.created_list) < self.max_num:
            self.create_thread()
        task = (func, args, callback,)
        self.q.put(task)

    def create_thread(self):
        """
        创建一个线程
        """
        t = threading.Thread(target=self.call)
        t.setDaemon(self.Deamon)
        t.start()
        self.created_list.append(t)  # 将当前线程加入已创建线程列表created_list

    def call(self):
        """
        循环去获取任务函数并执行任务函数
        """
        current_thread = threading.current_thread()  # 获取当前线程对象·
        event = self.q.get()  # 从任务队列获取任务
        while event != self.StopEvent:  # 判断获取到的任务是否是终止符

            func, arguments, callback = event  # 从任务中获取函数名、参数、和回调函数名
            try:
                result = func(*arguments)
                func_excute_status = True  # func执行成功状态
            except Exception as e:
                func_excute_status = False
                result = None
                print
                '函数执行产生错误', e  # 打印错误信息

            if func_excute_status:  # func执行成功后才能执行回调函数
                if callback is not None:  # 判断回调函数是否是空的
                    try:
                        callback(result)
                    except Exception as e:
                        print
                        '回调函数执行产生错误', e  # 打印错误信息

            with self.worker_state(self.free_list, current_thread):
                # 执行完一次任务后，将线程加入空闲列表。然后继续去取任务，如果取到任务就将线程从空闲列表移除
                if self.terminal:  # 判断线程池终止命令，如果需要终止，则使下次取到的任务为StopEvent。
                    event = self.StopEvent
                else:  # 否则继续获取任务
                    event = self.q.get()  # 当线程等待任务时，q.get()方法阻塞住线程，使其持续等待

        else:  # 若线程取到的任务是终止符，就销毁线程
            # 将当前线程从已创建线程列表created_list移除
            self.created_list.remove(current_thread)

    def close(self):
        """
        执行完所有的任务后，所有线程停止
        """
        full_size = len(self.created_list)  # 按已创建的线程数量往线程队列加入终止符。
        while full_size:
            self.q.put(self.StopEvent)
            full_size -= 1

    def terminate(self):
        """
        无论是否还有任务，终止线程
        """
        self.terminal = True
        while self.created_list:
            self.q.put(self.StopEvent)

        self.q.queue.clear()  # 清空任务队列

    def join(self):
        """
        阻塞线程池上下文，使所有线程执行完后才能继续
        """
        for t in self.created_list:
            t.join()

    @contextlib.contextmanager  # 上下文处理器，使其可以使用with语句修饰
    def worker_state(self, state_list, worker_thread):
        """
        用于记录线程中正在等待的线程数
        """
        state_list.append(worker_thread)
        try:
            yield
        finally:
            state_list.remove(worker_thread)



import serial.tools.list_ports


def getSerials():
    """
    查询串口列表
    :return: 串口list
    """
    port_list = list(serial.tools.list_ports.comports())
    if len(port_list) == 0:
        return []
    else:
        return [port_list[i].device for i in range(0, len(port_list))]


from time import *


def secondToDhms(second):
    """
    秒转换成天、时、分、秒格式
    :param second:秒
    :return:天、时、分、秒格式字符串
    """
    m, s = divmod(second, 60)
    h, m = divmod(m, 60)
    d, h = divmod(h, 24)
    return str("%02d:%02d:%02d:%02d" % (d, h, m, s))


def secondToHms(second):
    """
    秒转换成时、分、秒格式
    :param second: 秒
    :return: 时、分、秒格式字符串
    """
    m, s = divmod(second, 60)
    h, m = divmod(m, 60)
    return str("%02d:%02d:%02d" % (h, m, s))


def nowTimeStr():
    """
    当前时间格式化
    :return: 当前时间格式化后的字符串
    """
    return strftime("%Y-%m-%d %H:%M:%S", localtime())


# def nowTimeSecond():
#     """
#     当前时间的秒数
#     :rtype: object
#     """
#     return int(mktime(strptime(nowTimeStr(), "%Y-%m-%d %H:%M:%S")))


def nowTimeSecond():
    """

    :return: 当前时间的秒数
    """
    return int(time())


def secondFormat(second):
    """
    给定秒数格式化
    :param second:秒
    :return: 格式化后的字符串
    """
    return strftime("%Y-%m-%d %H:%M:%S", localtime(second))


if __name__ == '__main__':
    print(int(time()))
    print(nowTimeSecond())



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetEasAlarm(Message):

    def __init__(self, alarmSwitch: int, filterData: int, start: int, hexContent: str, hexMask: str,
                 **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetEasAlarm.value
        self.alarmSwitch = alarmSwitch
        self.filterData = filterData
        self.start = start
        self.hexContent = hexContent
        self.hexMask = hexMask
        self.actionSuccess = kwargs.get("actionSuccess", None)  # type:ActionParamSuccess
        self.actionFail = kwargs.get("actionFail", None)  # type:ActionParamFail

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.alarmSwitch)
        buffer.putInt(self.filterData)
        buffer.putShort(self.start)
        if self.hexContent:
            content_bytes = hexToBytes(self.hexContent)
            buffer.putShort(len(content_bytes))
            buffer.putBytes(content_bytes)

        if self.hexMask:
            mask_bytes = hexToBytes(self.hexMask)
            buffer.putShort(len(mask_bytes))
            buffer.putBytes(mask_bytes)

        if self.actionSuccess is not None:
            buffer.putInt(0x01)
            suc_bytes = self.actionSuccess.toBytes()
            buffer.putShort(len(suc_bytes))
            buffer.putBytes(suc_bytes)

        if self.actionFail is not None:
            buffer.putInt(0x02)
            fail_bytes = self.actionFail.toBytes()
            buffer.putShort(len(fail_bytes))
            buffer.putBytes(fail_bytes)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Set failure."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from .enumg import EnumG
from .message import Message
from .parameter import Parameter
from .base_stop import MsgBaseStop
from .log_base_epc_info import LogBaseEpcInfo
from .app_get_readerInfo import MsgAppGetReaderInfo
from .app_get_baseVersion import MsgAppGetBaseVersion
from .app_set_serialParam import MsgAppSetSerialParam
from .app_get_serialParam import MsgAppGetSerialParam
from .app_set_gpo import MsgAppSetGpo
from .app_get_gpiState import MsgAppGetGpiState
from .app_set_ethernetIp import MsgAppSetEthernetIP
from .app_get_ethernetIp import MsgAppGetEthernetIP
from .app_get_readerMac import MsgAppGetReaderMac
from .app_set_tpcMode import MsgAppSetTcpMode
from .app_get_tcpMode import MsgAppGetTcpMode
from .app_get_gpiTrigger import MsgAppGetGpiTrigger
from .app_set_gpiTrigger import MsgAppSetGpiTrigger
from .app_set_wiegand import MsgAppSetWiegand
from .app_get_wiegand import MsgAppGetWiegand
from .app_reset import MsgAppReset
from .app_heartbeat import MsgAppHeartbeat
from .app_set_readerTime import MsgAppSetReaderTime
from .app_get_readerTime import MsgAppGetReaderTime
from .app_set_rs485 import MsgAppSetRs485
from .app_get_rs485 import MsgAppGetRs485
from .app_restoreDefault import MsgAppRestoreDefault
from .app_set_breakpointResume import MsgAppSetBreakpointResume
from .app_get_breakpointResume import MsgAppGetBreakpointResume
from .app_get_cacheTagData import MsgAppGetCacheTagData
from .app_clearCacheData import MsgAppClearCacheData
from .app_tagDataReply import MsgAppTagDataReply
from .app_set_beepOnOff import MsgAppSetBeepOnOff
from .app_set_beep import MsgAppSetBeep
from .app_get_whiteList import MsgAppGetWhiteList
from .app_import_whiteList import MsgAppImportWhiteList
from .app_del_whiteList import MsgAppDelWhiteList
from .app_set_whiteListAction import MsgAppSetWhiteListAction
from .app_get_whiteListAction import MsgAppGetWhiteListAction
from .app_get_whiteListSwitch import MsgAppGetWhiteListSwitch
from .app_set_whiteListSwitch import MsgAppSetWhiteListSwitch
from .app_set_udpParam import MsgAppSetUdpParam
from .app_get_udpParam import MsgAppGetUdpParam
from .app_set_httpParam import MsgAppSetHttpParam
from .app_get_httpParam import MsgAppGetHttpParam
from .app_wifiHotspotSearch import MsgAppSetWifiHotspotSearch
from .app_get_wifiHotspotSearch import MsgAppGetWifiHotspotSearch
from .app_set_wifiHotspot import MsgAppSetWifiHotspot
from .app_get_wifiConnectStatus import MsgAppGetWifiConnectStatus
from .app_set_wifiIp import MsgAppSetWifiIp
from .app_get_wifiIp import MsgAppGetWifiIp
from .app_set_wifiSwitch import MsgAppSetWifiSwitch
from .app_get_wifiSwitch import MsgAppGetWifiSwitch
from .action_param_success import ActionParamSuccess
from .action_param_fail import ActionParamFail
from .app_set_easAlarm import MsgAppSetEasAlarm
from .app_get_easAlarm import MsgAppGetEasAlarm
from .base_get_capabilities import MsgBaseGetCapabilities
from .base_set_power import MsgBaseSetPower
from .base_get_power import MsgBaseGetPower
from .base_set_freRange import MsgBaseSetFreqRange
from .base_get_freRange import MsgBaseGetFreqRange
from .base_set_frequency import MsgBaseSetFrequency
from .base_get_frequency import MsgBaseGetFrequency
from .base_set_antennaHub import MsgBaseSetAntennaHub
from .base_get_antennaHub import MsgBaseGetAntennaHub
from .base_set_tagLog import MsgBaseSetTagLog
from .base_get_tagLog import MsgBaseGetTagLog
from .base_get_baseband import MsgBaseGetBaseband
from .base_set_baseband import MsgBaseSetBaseband
from .base_set_autoDormancy import MsgBaseSetAutoDormancy
from .base_get_autoDormancy import MsgBaseGetAutoDormancy
from .base_set_residenceTime import MsgBaseSetResidenceTime
from .base_get_residenceTime import MsgBaseGetResidenceTime
from .param_epc_filter import ParamEpcFilter
from .param_epc_readTid import ParamEpcReadTid
from .param_epc_readUserData import ParamEpcReadUserData
from .param_epc_readReserved import ParamEpcReadReserved
from .param_epc_readEpc import ParamEpcReadEpc
from .param_epc_fastId import ParamEpcFastId
from .base_inventoryEpc import MsgBaseInventoryEpc
from .base_writeEpc import MsgBaseWriteEpc
from .base_lockEpc import MsgBaseLockEpc
from .base_destroyEpc import MsgBaseDestroyEpc
from .base_writeMonzaQt import MsgBaseWriteMonzaQt
from .base_super_rw import MsgBaseSuperRW
from .param_6b_readUserData import Param6bReadUserData
from .base_inventory6b import MsgBaseInventory6b
from .log_base_6b_info import LogBase6bInfo
from .log_base_epc_over import LogBaseEpcOver
from .log_base_6b_over import LogBase6bOver
from .log_base_gb_info import LogBaseGbInfo
from .log_base_gb_over import LogBaseGbOver
from .base_write6b import MsgBaseWrite6b
from .base_lock6b import MsgBaseLock6b
from .base_lock6bGet import MsgBaseLock6bGet
from .param_gb_readUserData import ParamGbReadUserData
from .base_inventoryGb import MsgBaseInventoryGb
from .base_writeGb import MsgBaseWriteGb
from .base_lockGb import MsgBaseLockGb
from .base_destroyGb import MsgBaseDestroyGb
from .base_safe_attestation import MsgBaseSafeAttestation
from .param_safe_attestation import ParamSafeEncipheredData
from .base_inventory_Hybrid import MsgBaseInventoryHybrid
from .upgrade_app import MsgUpgradeApp
from .upgrade_baseband import MsgUpgradeBaseband
from .test_carrierWave import MsgTestCarrierWave
from .test_dcbias import MsgTestDCbias
from .test_dcbias_get import MsgTestDCbiasGet
from .test_power_calibration import MsgTestPowerCalibration
from .test_power_calibrationGet import MsgTestPowerCalibrationGet
from .test_vswr_check import MsgTestVSWRCheck
from .test_dc_autobias import MsgTestDCAutoBias
from .test_env_rssiDetection import MsgTestEnvRssiDetection
from .test_set_rssiCalibration import MsgTestSetRssiCalibration
from .test_get_rssiCalibration import MsgTestGetRssiCalibration
from .test_env_rssiAutoDetection import MsgTestEnvRssiAutoDetection
from .test_serialno_set import MsgTestSerialNoSet
from .test_workMode_set import MsgTestWorkModeSet
from .test_workMode_get import MsgTestWorkModeGet
from .test_encryptionParam_set import MsgTestEncryptionParamSet
from .test_encryptionParam_get import MsgTestEncryptionParamGet
from .test_r2000_readWrite import MsgTestR2000ReadWrite
from .test_r2000_receivingGain import MsgTestReceivingGain
from .app_gpiStart import LogGpiStart
from .app_gpiOver import LogGpiOver
from .base_set_gb_baseband import MsgBaseSetGbBaseBand
from .base_get_gb_baseband import MsgBaseGetGbBaseBand
from .base_inventoryGJb import MsgBaseInventoryGJb
from .base_writeGJb import MsgBaseWriteGJb
from .base_lockGJb import MsgBaseLockGJb
from .base_destroyGJb import MsgBaseDestroyGJb
from .app_all_gpiState import LogAppAllGpiState
from .log_base_gjb_info import LogBaseGJbInfo
from .log_base_gjb_over import LogBaseGJbOver

__all__ = ["Message", "Parameter", "MsgBaseStop", "LogBaseEpcInfo", "EnumG", "MsgAppGetReaderInfo",
           "MsgAppGetBaseVersion", "MsgAppSetSerialParam", "MsgAppGetSerialParam", "MsgAppSetGpo", "MsgAppGetGpiState",
           "MsgAppSetEthernetIP", "MsgAppGetEthernetIP", "MsgAppGetReaderMac", "MsgAppSetTcpMode", "MsgAppGetTcpMode",
           "MsgAppGetGpiTrigger", "MsgAppSetGpiTrigger", "MsgAppSetWiegand", "MsgAppGetWiegand", "MsgAppReset",
           "MsgAppSetReaderTime", "MsgAppGetReaderTime", "MsgAppHeartbeat", "MsgAppSetRs485", "MsgAppGetRs485",
           "MsgAppRestoreDefault", "MsgAppSetBreakpointResume", "MsgAppGetBreakpointResume", "MsgAppGetCacheTagData",
           "MsgAppClearCacheData", "MsgAppTagDataReply", "MsgAppSetBeepOnOff", "MsgAppSetBeep", "MsgAppGetWhiteList",
           "MsgAppImportWhiteList", "MsgAppDelWhiteList", "MsgAppSetWhiteListAction", "MsgAppGetWhiteListAction",
           "MsgAppGetWhiteListSwitch", "MsgAppSetWhiteListSwitch", "MsgAppSetUdpParam", "MsgAppGetUdpParam",
           "MsgAppSetHttpParam", "MsgAppGetHttpParam", "MsgAppSetWifiHotspotSearch", "MsgAppGetWifiHotspotSearch",
           "MsgAppSetWifiHotspot", "MsgAppGetWifiConnectStatus", "MsgAppSetWifiIp", "MsgAppGetWifiIp",
           "MsgAppSetWifiSwitch", "MsgAppGetWifiSwitch", "ActionParamSuccess", "ActionParamFail", "MsgAppSetEasAlarm",
           "MsgAppGetEasAlarm", "MsgBaseGetCapabilities", "MsgBaseSetPower", "MsgBaseGetPower", "MsgBaseSetFreqRange",
           "MsgBaseGetFreqRange", "MsgBaseSetFrequency", "MsgBaseGetFrequency", "MsgBaseSetAntennaHub",
           "MsgBaseGetAntennaHub", "MsgBaseSetTagLog", "MsgBaseGetTagLog", "MsgBaseGetBaseband", "MsgBaseSetBaseband",
           "MsgBaseSetAutoDormancy", "MsgBaseGetAutoDormancy", "MsgBaseSetResidenceTime", "MsgBaseGetResidenceTime",
           "ParamEpcFilter", "ParamEpcReadTid", "ParamEpcReadUserData", "ParamEpcReadEpc", "ParamEpcReadReserved",
           "ParamEpcFastId", "MsgBaseInventoryEpc", "MsgBaseWriteEpc", "MsgBaseLockEpc", "MsgBaseDestroyEpc",
           "MsgBaseWriteMonzaQt", "MsgBaseSuperRW", "Param6bReadUserData", "MsgBaseInventory6b", "LogBase6bInfo",
           "LogBaseEpcOver", "LogBase6bOver", "LogBaseGbInfo", "LogBaseGbOver", "MsgBaseWrite6b", "MsgBaseLock6b",
           "MsgBaseLock6bGet", "ParamGbReadUserData", "MsgBaseInventoryGb", "MsgBaseWriteGb", "MsgBaseLockGb",
           "MsgBaseDestroyGb", "ParamSafeEncipheredData", "MsgBaseSafeAttestation", "MsgBaseInventoryHybrid",
           "MsgUpgradeApp", "MsgUpgradeBaseband", "MsgTestCarrierWave", "MsgTestDCbiasGet", "MsgTestDCbias",
           "MsgTestPowerCalibration", "MsgTestPowerCalibrationGet", "MsgTestVSWRCheck", "MsgTestDCAutoBias",
           "MsgTestEnvRssiDetection", "MsgTestGetRssiCalibration", "MsgTestSetRssiCalibration", "MsgTestSerialNoSet",
           "MsgTestWorkModeSet", "MsgTestEnvRssiAutoDetection", "MsgTestWorkModeGet", "MsgTestEncryptionParamSet",
           "MsgTestEncryptionParamGet", "MsgTestR2000ReadWrite", "MsgTestReceivingGain", "LogGpiStart", "LogGpiOver",
           "base_set_gb_baseband", "base_get_gb_baseband","MsgBaseInventoryGJb","MsgBaseWriteGJb","MsgBaseLockGJb",
           "MsgBaseDestroyGJb","MsgBaseGetGbBaseBand","MsgBaseSetGbBaseBand","LogAppAllGpiState","LogBaseGJbInfo","LogBaseGJbOver"]



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseGetPower(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_GetPower.value
        self.dicPower = {}

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            for i in range(int(len(self.cData) / 2)):
                key = buffer.readInt()
                value = buffer.readInt()
                self.dicPower[key] = value
            self.rtCode = 0

    def __str__(self) -> str:
        return str(self.dicPower)



from uhf.reader.utils.decodeUtils import *
from uhf.reader.utils.HexUtils import *
from uhf.reader.utils import *
from uhf.reader.utils.pcUtils import *
from .ring_buffer import RingBuffer
from .dateUtils import *
from .serial_utils import *
from .usb_utils import *
from .hid_utils import *

__all__ = ["DynamicBuffer", "listToAscii", "bytesToAscii", "RingBuffer", "bytesToHex", "hexToBytes", "secondToDhms",
           "secondToHms", "nowTimeStr", "nowTimeSecond", "secondFormat", "getPcLen", "getPcValue", "getGbPcValue",
           "getSerials",  "getEpcData", "getGbData", ]
# "getUsbHidPathDict","getUsbHidPathList"
# "getUsbHidDeviceDict", "getUsbHidDeviceList",


from uhf.reader.protocol import *
from .enumg import EnumG


class MsgBaseStop(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_Stop.value

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Stop failure"}
            self.rtCode = self.cData[0]
            self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseLock6b(Message):

    def __init__(self, antennaEnable: int, hexMatchTid: str, lockIndex: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_Lock6b.value
        self.antennaEnable = antennaEnable
        self.hexMatchTid = hexMatchTid
        self.lockIndex = lockIndex
        self.errorIndex = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaEnable)
        buffer.putBytes(hexToBytes(self.hexMatchTid))
        buffer.putInt(self.lockIndex)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Other error.", }
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)
            if len(self.cData) > 1:
                errBuffer = DynamicBuffer("0x" + bytesToHex(self.cData))
                errBuffer.pos = 8
                if errBuffer.readInt() == 1:
                    self.errorIndex = errBuffer.readInt()



# from uhf.reader import *
# import logging
# from time import *
#
#
# def receivedEpc(epcInfo: LogBaseEpcInfo):
#     if epcInfo.result == 0:
#         pass
#         print(epcInfo.epc)
#
#
# def uploadEpcOver(epcOver: LogBaseEpcOver):
#     print("LogBaseEpcOver")
#
#
# def tcpDisconnect(readName: (),client):
#     print("触发断链--》", readName)

#
# def gpiStart(start: LogGpiStart):
#     pass
#
#
# def gpiOver(over: LogGpiOver):
#     pass
#
#
# def clientConnected(readerSerialNumber: str, gClient: GClient):
#     pass
#
#
# # logging.basicConfig(format='%(asctime)s - %(pathname)s[line:%(lineno)d] '
# #                            '- %(levelname)s: %(message)s', level=logging.INFO)
#
# if __name__ == '__main__':
#     client = GClient()
#     # 串口连接 True 即成功连接
#     # if client.openSerial(("COM7", 115200)):  # 不要重复连接，可调用close释放资源再次连接，未close的client可复用
#     # if client.openUsbHid(getUsbHidPathList()[0]):
#     if client.openTcp(("192.168.1.168", 8160)):
#         # 订阅盘点回调
#         # client.printLog = False
#         client.callTcpDisconnect = tcpDisconnect
#         client.callEpcInfo = receivedEpc
#         client.callEpcOver = uploadEpcOver
#
#         msg = MsgAppSetGpo()
#         msg.gpo1 = 1  # 1-高 0-低
#         if client.sendSynMsg(msg) == 0:
#             print("发送成功")
#
#
#         # client.callTcpDisconnect = tcpDisconnect
#         # client.callGpiStart = gpiStart
#         # client.callGpiOver = gpiOver
#         # 盘点6c 1号天线 循环盘点
#         # while True:
#         #     s = input()
#         #     # print(type(s))
#         #     if s == "1":
#         #         msg = MsgBaseInventoryEpc(EnumG.AntennaNo_1.value, EnumG.InventoryMode_Inventory.value)
#         #         # msg.readEpc = ParamEpcReadEpc(2, 6)
#         #         # msg.readTid = ParamEpcReadTid(0, 6)
#         #         # msg.readUserData = ParamEpcReadUserData(0, 6)
#         #
#         #         # for i in range(20):
#         #         # logging.info("sendSynMsg")
#         #         client.sendSynMsg(msg)
#         #         # print(msg.rtMsg)
#         #         # logging.info(msg.rtMsg)
#         #     else:
#         #         client.close()
#
#         # sleep(10)  # 盘点5s
#
#         # stop = MsgBaseStop()  # 停止盘点
#         # if client.sendSynMsg(stop) == 0:
#         #     print(stop.rtMsg)
#
#         # data = "FFFFFF2222233333333333333333311111111111100FFFFFFFFFF2222233333333333333333311111111111100FFFFFFFFFFFFFFFFFFFFF3FFFFFFFFFFFFFF33"
#         # value = getEpcData(data)
#         #
#         # print(value)
#         # epc = MsgBaseWriteEpc(1, EnumG.WriteArea_UserData.value, 0, data)
#         # client.sendSynMsg(epc)
#         # print(epc.rtMsg)
#
#         # stop = MsgBaseStop()  # 停止盘点
#         # if client.sendSynMsg(stop) == 0:
#         #     print(stop.rtMsg)
#
#         # client.close()  # 断开连接 释放资源
#
#         # g_server = GServer()
#         # g_server.callGClientConnected = clientConnected



from .tcp_client import TcpClient
from .serial_client import SerialClient
# from .usb_hid import UsbHidClient
# from .hid import HidClient
from uhf.reader.protocol import *
from uhf.reader.utils import *


# import usb.core


class GClient(object):

    def __init__(self):
        self.__ci = None
        self.__msgDic = {}
        self.readerName = None
        self.__serialNumber = None
        self.__callEpcInfo = None
        self.__callEpcOver = None
        self.__call6bInfo = None
        self.__call6bOver = None
        self.__callGbInfo = None
        self.__callGbOver = None
        self.__callGJbInfo = None
        self.__callGJbOver = None
        self.__callTcpDisconnect = None
        self.__callGpiStart = None
        self.__callGpiOver = None
        self.__callCacheDataOver = None
        self.__allGpiState = None
        self.printLog = True

    def openTcp(self, readName: (), **kwargs):
        self.__ci = TcpClient()
        if self.__ci.open(readName, **kwargs):
            self.readerName = readName
            self.__ci.onMessageReceived = self.processMessage
            self.__ci.onTcpDisConnect = self.__callTcpDisConnectEvent
            return True
        return False

    def __callTcpDisConnectEvent(self, addr):
        if self.__callTcpDisconnect:
            self.__callTcpDisconnect(addr, self)

    def openServer(self, tcpClient: TcpClient):
        if not tcpClient:
            return False
        self.__ci = tcpClient
        self.readerName = tcpClient.addr
        self.__ci.onMessageReceived = self.processMessage
        self.__ci.onTcpDisConnect = self.__callTcpDisConnectEvent
        return True

    def openSerial(self, readName: (), **kwargs):
        if len(readName) == 2:
            self.__ci = SerialClient()
            if self.__ci.open(readName, **kwargs):
                self.readerName = readName
                self.__ci.onMessageReceived = self.processMessage
                return True
        return False

    def openSerial485(self, readName: (), **kwargs):
        if len(readName) == 3:
            self.__ci = SerialClient()
            if self.__ci.open(readName, **kwargs):
                self.__ci.rs485Address = readName[2]
                self.__ci.isRs485 = True
                self.readerName = readName
                self.__ci.onMessageReceived = self.processMessage
                return True
        return False

    # def openUsbHidDevice(self, dev: usb.core.Device):
    #     self.__ci = UsbHidClient()
    #     if self.__ci.open(dev):
    #         self.__ci.onMessageReceived = self.processMessage
    #         return True
    #     return False

    # def openUsbHid(self, param: bytes):
    #     self.__ci = HidClient()
    #     if self.__ci.open(param):
    #         self.readerName = param
    #         self.__ci.onMessageReceived = self.processMessage
    #         return True
    #     return False

    def sendSynMsg(self, msg: Message, timeout=3):
        if self.__ci:
            self.__msgDic[msg.toKey()] = None
            self.__ci.sendMsg(msg)
            if self.printLog:
                print("send -> ", bytesToHex(msg.msgData))
            with self.__ci.lock:
                # print("开始等待")
                self.__ci.lock.wait(timeout)
            data = self.__msgDic.get(msg.toKey(), None)
            if data:
                if self.printLog:
                    print("receive->", bytesToHex(data.msgData))
                msg.cData = data.cData
                msg.unPack()
                return msg.rtCode

    def sendSynBytes(self, bytes, timeout=3):
        if self.__ci:
            return self.sendSynMsg(Message().new(bytesToHex(bytes)), timeout)

    def senUnSynMsg(self, msg: Message):
        if self.__ci:
            self.__ci.sendMsg(msg)

    def senUnSynBytes(self, bytes):
        if self.__ci:
            self.__ci.sendBytes(bytes)

    def processMessage(self, msg: Message):
        if msg.mt_12 == 0:
            # print("收发指令", msg.msgId)
            if msg.toKey() in self.__msgDic:
                self.__msgDic[msg.toKey()] = msg
                with self.__ci.lock:
                    # print("结束等待")
                    self.__ci.lock.notify()
        else:
            if msg.mt_8_11 == EnumG.Msg_Type_Bit_Base.value:
                if msg.msgId == EnumG.BaseLogMid_Epc.value:
                    if self.__callEpcInfo:
                        info = LogBaseEpcInfo()
                        info.cData = msg.cData
                        info.unPack()
                        if self.__serialNumber:
                            info.readerSerialNumber = self.__serialNumber
                        info.readerName = self.readerName
                        self.__callEpcInfo(info)
                elif msg.msgId == EnumG.BaseLogMid_EpcOver.value:
                    if self.__callEpcOver:
                        over = LogBaseEpcOver()
                        over.cData = msg.cData
                        over.unPack()
                        if self.__serialNumber:
                            over.readerSerialNumber = self.__serialNumber
                        over.readerName = self.readerName
                        self.__callEpcOver(over)
                elif msg.msgId == EnumG.BaseLogMid_6b.value:
                    if self.__call6bInfo:
                        b_info = LogBase6bInfo()
                        b_info.cData = msg.cData
                        b_info.unPack()
                        if self.__serialNumber:
                            b_info.readerSerialNumber = self.__serialNumber
                        b_info.readerName = self.readerName
                        self.__call6bInfo(b_info)
                elif msg.msgId == EnumG.BaseLogMid_6bOver.value:
                    if self.__call6bOver:
                        b_over = LogBase6bOver()
                        b_over.cData = msg.cData
                        b_over.unPack()
                        if self.__serialNumber:
                            b_over.readerSerialNumber = self.__serialNumber
                        b_over.readerName = self.readerName
                        self.__call6bOver(b_over)
                elif msg.msgId == EnumG.BaseLogMid_Gb.value:
                    if self.__callGbInfo:
                        gb_info = LogBaseGbInfo()
                        gb_info.cData = msg.cData
                        gb_info.unPack()
                        if self.__serialNumber:
                            gb_info.readerSerialNumber = self.__serialNumber
                        gb_info.readerName = self.readerName
                        self.__callGbInfo(gb_info)
                elif msg.msgId == EnumG.BaseLogMid_GbOver.value:
                    if self.__callGbOver:
                        gb_over = LogBaseGbOver()
                        gb_over.cData = msg.cData
                        gb_over.unPack()
                        if self.__serialNumber:
                            gb_over.readerSerialNumber = self.__serialNumber
                        gb_over.readerName = self.readerName
                        self.__callGbOver(gb_over)
                elif msg.msgId == EnumG.BaseLogMid_GJb.value:
                    if self.__callGJbInfo:
                        gjb_info = LogBaseGJbInfo()
                        gjb_info.cData = msg.cData
                        gjb_info.unPack()
                        if self.__serialNumber:
                            gjb_info.readerSerialNumber = self.__serialNumber
                        gjb_info.readerName = self.readerName
                        self.__callGJbInfo(gjb_info)
                elif msg.msgId == EnumG.BaseLogMid_GJbOver.value:
                    if self.__callGJbOver:
                        gjb_over = LogBaseGJbOver()
                        gjb_over.cData = msg.cData
                        gjb_over.unPack()
                        if self.__serialNumber:
                            gjb_over.readerSerialNumber = self.__serialNumber
                        gjb_over.readerName = self.readerName
                        self.__callGJbOver(gjb_over)
            elif msg.mt_8_11 == EnumG.Msg_Type_Bit_App.value:
                if msg.msgId == EnumG.AppMid_Heartbeat.value:
                    if self.printLog:
                        print("[heartbeat]", bytesToHex(msg.msgData))
                    self.senUnSynBytes(msg.msgData)
                if msg.msgId == EnumG.AppLogMid_gpi.value:
                    if self.__callGpiStart:
                        start = LogGpiStart()
                        start.cData = msg.cData
                        start.unPack()
                        if self.__serialNumber:
                            start.readerSerialNumber = self.__serialNumber
                        start.readerName = self.readerName
                        self.__callGpiStart(start)
                elif msg.msgId == EnumG.AppLogMid_gpiOver.value:
                    if self.__callGpiOver:
                        over = LogGpiOver()
                        over.cData = msg.cData
                        over.unPack()
                        if self.__serialNumber:
                            over.readerSerialNumber = self.__serialNumber
                        over.readerName = self.readerName
                        self.__callGpiOver(over)
                elif msg.msgId == EnumG.AppLogMid_allGpiState.value:
                    if self.__allGpiState:
                        state = LogAppAllGpiState()
                        state.cData = msg.cData
                        state.unPack()
                        if self.__serialNumber:
                            state.readerSerialNumber = self.__serialNumber
                        state.readerName = self.readerName
                        self.__allGpiState(state)
                elif msg.msgId == EnumG.AppMid_GetCacheTagData.value:
                    if self.__callCacheDataOver:
                        data = MsgAppGetCacheTagData()
                        data.cData = msg.cData
                        data.unPack()
                        self.__callCacheDataOver(data)

    @property
    def callEpcInfo(self):
        return self.__callEpcInfo

    @callEpcInfo.setter
    def callEpcInfo(self, callEpc):
        # if self.ci:
        self.__callEpcInfo = callEpc

    @property
    def callEpcOver(self):
        return self.__callEpcOver

    @callEpcOver.setter
    def callEpcOver(self, callEpcOver):
        # if self.ci:
        self.__callEpcOver = callEpcOver

    @property
    def call6bInfo(self):
        return self.__call6bInfo

    @call6bInfo.setter
    def call6bInfo(self, call6b):
        # if self.ci:
        self.__call6bInfo = call6b

    @property
    def call6bOver(self):
        return self.__call6bOver

    @call6bOver.setter
    def call6bOver(self, call6bOver):
        # if self.ci:
        self.__call6bOver = call6bOver

    @property
    def callGbInfo(self):
        return self.__callGbInfo

    @callGbInfo.setter
    def callGbInfo(self, callGb):
        # if self.ci:
        self.__callGbInfo = callGb

    @property
    def callGbOver(self):
        return self.__callGbOver

    @callGbOver.setter
    def callGbOver(self, callGbOver):
        # if self.ci:
        self.__callGbOver = callGbOver

    @property
    def callGJbInfo(self):
        return self.__callGJbInfo

    @callGJbInfo.setter
    def callGJbInfo(self, callGJb):
        # if self.ci:
        self.__callGJbInfo = callGJb

    @property
    def callGJbOver(self):
        return self.__callGJbOver

    @callGJbOver.setter
    def callGJbOver(self, callGJbOver):
        # if self.ci:
        self.__callGJbOver = callGJbOver

    @property
    def serialNumber(self):
        return self.__serialNumber

    @serialNumber.setter
    def serialNumber(self, serialNumber):
        # if self.ci:
        self.__serialNumber = serialNumber

    @property
    def callTcpDisconnect(self):
        return self.__callTcpDisconnect

    @callTcpDisconnect.setter
    def callTcpDisconnect(self, callDis):
        # if self.ci and isinstance(self.ci, TcpClient):
        self.__callTcpDisconnect = callDis
        # self.ci.callDisconnected = callDis

    @property
    def callGpiStart(self):
        return self.__callGpiStart

    @callGpiStart.setter
    def callGpiStart(self, callGpi):
        self.__callGpiStart = callGpi

    @property
    def callGpiOver(self):
        return self.__callGpiOver

    @callGpiOver.setter
    def callGpiOver(self, callGpi):
        self.__callGpiOver = callGpi

    @property
    def callCacheDataOver(self):
        return self.__callCacheDataOver

    @callCacheDataOver.setter
    def callCacheDataOver(self, cacheOver):
        self.__callCacheDataOver = cacheOver

    @property
    def allGpiState(self):
        return self.__allGpiState

    @allGpiState.setter
    def allGpiState(self, callGpi):
        self.__allGpiState = callGpi

    def close(self):
        if self.__ci:
            self.__ci.close()
            self.__ci = None
            self.__msgDic.clear()
            self.readerName = None
            self.__serialNumber = None
            self.__callEpcInfo = None
            self.__callEpcOver = None
            self.__call6bInfo = None
            self.__call6bOver = None
            self.__callGbInfo = None
            self.__callGbOver = None
            self.__callGJbInfo = None
            self.__callGJbOver = None
            self.__callTcpDisconnect = None
            self.__callGpiStart = None
            self.__callGpiOver = None
            self.__callCacheDataOver = None
            self.__allGpiState = None



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetEasAlarm(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetEasAlarm.value
        self.alarmSwitch = None
        self.filterArea = None
        self.start = None
        self.hexContent = None
        self.hexMask = None
        self.actionSuccess = None  # type:ActionParamSuccess
        self.actionFail = None  # type:ActionParamFail

    def bytesToClass(self):
        pass

    def pack(self):
        pass

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.alarmSwitch = buffer.readInt()
            self.filterArea = buffer.readInt()
            self.start = buffer.readShort()
            dataLen = buffer.readShort()
            if dataLen:
                self.hexContent = bytesToHex(buffer.readBytes(dataLen * 8))
            maskLen = buffer.readShort()
            if maskLen:
                self.hexMask = bytesToHex(buffer.readBytes(maskLen * 8))
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    sucLen = buffer.readShort()
                    if sucLen:
                        suc_bytes = buffer.readBytes(sucLen * 8)
                        self.actionSuccess = ActionParamSuccess(0).bytesToClass(suc_bytes)
                if pid == 2:
                    failLen = buffer.readShort()
                    if failLen:
                        fail_bytes = buffer.readBytes(failLen * 8)
                        self.actionFail = ActionParamSuccess(0).bytesToClass(fail_bytes)
            self.rtCode = 0

    def __str__(self) -> str:
        return str(
            (self.alarmSwitch, self.filterArea, self.start, self.hexContent, self.hexMask, self.actionSuccess.__str__(),
             self.actionFail.__str__()))



from uhf.reader.utils.HexUtils import *
from uhf.reader.utils.byteBuffer import *
from uhf.reader.protocol import EnumG

class Message(object):

    def __init__(self):
        # self.head = 0x5A
        self.head = EnumG.Msg_Header.value
        self.pType = 0x00
        self.pVersion = 0x01
        self.mt_14_15 = "00"
        self.mt_13 = "0"
        self.mt_12 = "0"
        self.mt_8_11 = 0
        self.msgId = 0xFF

        self.rs485Address = 0
        self.dataLen = 0
        self.cData = []
        self.crcData = []
        self.crc = []
        self.msgData = []
        self.rtCode = -1
        self.rtMsg = None

    def new(self, data: str):
        try:
            if not data.startswith("0x"):
                data = "0x" + data
            bitBuffer = DynamicBuffer(data)
            self.msgData = bitBuffer.tobytes()
            self.head = bitBuffer.readInt()
            self.pType = bitBuffer.readInt()
            self.pVersion = bitBuffer.readInt()
            self.mt_14_15 = bitBuffer.readBitLen(2)
            self.mt_13 = bitBuffer.readBitLen(1)
            self.mt_12 = bitBuffer.readBitLen(1)
            self.mt_8_11 = bitBuffer.readBitLen(4)
            self.msgId = bitBuffer.readInt()
            if self.mt_13 == 1:
                self.rs485Address = bitBuffer.readInt()
            self.dataLen = bitBuffer.readShort()
            if self.dataLen:
                self.cData = bitBuffer.readBytes(self.dataLen * 8)
            self.crc = bitBuffer.readBytes(16)
            bitBuffer.setPos(8)
            self.crcData = bitBuffer.readBytes((len(self.msgData) - 3) * 8)
            return self
        except Exception as ex:
            print(["Message-->"], ex.args)
            # raise ex
            # return None

    def toByte(self, is485):
        try:
            bitBuffer = DynamicBuffer()
            bitBuffer.putInt(self.head).putInt(self.pType).putInt(self.pVersion)
            bitBuffer.putString(4, str(self.mt_14_15) + str(self.mt_13) + str(self.mt_12))
            bitBuffer.putString(4, self.mt_8_11).putInt(self.msgId)
            if is485:
                bitBuffer.putInt(self.rs485Address)
            bitBuffer.putShort(self.dataLen)
            if self.cData and len(self.cData) == self.dataLen:
                bitBuffer.putBytes(self.cData)
            self.crcData = Message.crc16_Xmodem_hex(bitBuffer.hex[2:]).zfill(4)  # 当前hex去掉5a
            self.crc = hexToBytes(self.crcData)
            bitBuffer.putBytes(self.crc)
            self.msgData = bitBuffer.tobytes()
            # print(bytesToHex(self.msgData))
            return self.msgData
        except Exception as ex:
            raise ex

    def pack(self):
        pass

    def unPack(self):
        pass

    def toKey(self):
        return str(self.mt_8_11) + str(self.msgId)

    def checkCrc(self):
        if self.crcData and self.crc:
            xmodem_hex = Message.crc16_Xmodem_hex(self.crcData)
            to_hex = bytesToHex(self.crc)
            if xmodem_hex.zfill(4) == to_hex:
                return True
            return False

    @staticmethod
    def crc16_Xmodem(bytes):
        if isinstance(bytes, str):
            bytes = hexToBytes(bytes)
        crc = 0x00
        polynomial = 0x1021
        for byte in bytes:
            for i in range(8):
                bit = ((byte >> (7 - i) & 1) == 1)
                c15 = ((crc >> 15 & 1) == 1)
                crc <<= 1
                if c15 ^ bit:
                    crc ^= polynomial
        crc &= 0xffff
        return crc

    @staticmethod
    def crc16_Xmodem_hex(bytes):
        return hex(Message.crc16_Xmodem(bytes))[2:].zfill(4)

    @staticmethod
    def intToByte(value):
        return value.to_bytes(2, byteorder='big', signed=False)


if __name__ == '__main__':
    pass
    print(Message.crc16_Xmodem([0, 1, 17, 18, 0, 4, 0, 0, 1, 82]))
    print()
    print(hex(1628))
    print(bytesToHex(Message.intToByte(1628)))
    print("65c0".zfill(4))
    # msg = Message()
    # print(msg.toByte(False).hex())
    # msg.test([1])
    # if 1:
    #     print(124)
    # new = msg.new("0x5a000102ff0000885a")
    # print(new)
    # s1 = 34906
    # print(Message.crc16_Xmodem("000102FF0000"))
    # print(msg.calc_crc("000102FF0000"))
    # for key, value in enumerate(range(8)):
    #     print(key, value)
    # print(hex(Message.crc16_Xmodem("000102FF0000"))[2:])
    # print(Message.crc16_Xmodem_hex("000102FF0000"))
    # print(hexToBytes(s1.to_bytes(4, byteorder='big', signed=True).hex()))
    # temp=[90, 0, 1, 0, 255, 0, 0, 101, 50]
    # print(temp[1:len(temp)-2])
    # se = 20
    # print(hex(18))
    # print(se.to_bytes(4, byteorder='big', signed=False))
    # print("123"[-1])
    # print(int("2"))
    # print(str(bytesToInt("0000")) + str(bytesToInt("0001")))

    # msgType = str(bytesToInt(self.mt_14_15 + self.mt_13 + self.mt_12)) + str(bytesToInt(self.mt_8_11))
    # buffer = [self.head, self.pType, self.pVersion, int(msgType, 16), self.msgId]
    # if is485:
    #     buffer.append(self.rs485Address)
    # buffer.append(self.dataLen)
    # if self.cData and len(self.cData) == self.dataLen:
    #     buffer.append(self.cData)
    # self.crcData = Message.crc16_Xmodem_hex(buffer[1:])
    # self.crc = hexToBytes(self.crcData)
    # buffer.extend(self.crc)
    # self.msgData = buffer



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppImportWhiteList(Message):

    def __init__(self, packetNumber: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.__msgId = EnumG.AppMid_ImportWhiteList.value
        self.packetNumber = packetNumber
        self.packetContent = []

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.packetNumber)
        if self.packetContent:
            buffer.putShort(len(self.packetContent))
            buffer.putBytes(self.packetContent)
        else:
            buffer.putShort(0)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Fail."}
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.packetNumber = buffer.readLong()
            result = buffer.readInt()
            self.rtCode = result
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(result, None)



from bitstring import *

"""
Token examples: 
                        'int:12'    : 12 bits as a signed integer
                        'uint:8'    : 8 bits as an unsigned integer
                        'float:64'  : 8 bytes as a big-endian float
                        'intbe:16'  : 2 bytes as a big-endian signed integer
                        'uintbe:16' : 2 bytes as a big-endian unsigned integer
                        'intle:32'  : 4 bytes as a little-endian signed integer
                        'uintle:32' : 4 bytes as a little-endian unsigned integer
                        'floatle:64': 8 bytes as a little-endian float
                        'intne:24'  : 3 bytes as a native-endian signed integer
                        'uintne:24' : 3 bytes as a native-endian unsigned integer
                        'floatne:32': 4 bytes as a native-endian float
                        'hex:80'    : 80 bits as a hex string
                        'oct:9'     : 9 bits as an octal string
                        'bin:1'     : single bit binary string
                        'ue'        : next bits as unsigned exp-Golomb code
                        'se'        : next bits as signed exp-Golomb code
                        'uie'       : next bits as unsigned interleaved exp-Golomb code
                        'sie'       : next bits as signed interleaved exp-Golomb code
                        'bits:5'    : 5 bits as a bitstring
                        'bytes:10'  : 10 bytes as a bytes object
                        'bool'      : 1 bit as a bool
                        'pad:3'     : 3 bits of padding to ignore - returns None
"""


class DynamicBuffer(BitStream):

    # 重写init需要super
    # def __init__(self):
    #     super().__init__()

    def putInt(self, value):
        self.append("uint:8=%d" % value)
        return self

    def putShort(self, value):
        self.append("uint:16=%d" % value)
        return self

    def putLong(self, value):
        self.append("uint:32=%d" % value)
        return self

    def putString(self, bitLen, value):
        self.append("uint:%d=%s" % (bitLen, value))
        return self

    def putSigned(self, bitLen, value):
        self.append("int:%d=%s" % (bitLen, value))
        return self

    def putBytes(self, bytes):
        if bytes:
            for byte in bytes:
                self.putInt(byte)
        return self

    def addPos(self, bitLen):
        self.pos += bitLen

    def setPos(self, bitLen):
        self.pos = bitLen

    def readInt(self):
        return self.read("uint:8")

    def readShort(self):
        return self.read("uint:16")

    def readLong(self):
        return self.read("uint:32")

    def readBitLen(self, bitLen):
        return self.read("uint:%d" % bitLen)

    def readSigned(self, bitLen):
        return self.read("int:%d" % bitLen)

    def readBytes(self, bitLen):
        if bitLen:
            return [self.readInt() for bit in range(int(bitLen / 8))]


if __name__ == '__main__':
    print("int:8=", 5, sep="")

    buffer = DynamicBuffer()
    buffer.putString(1,1)
    buffer.putString(1,1)
    buffer.putString(4,0)
    buffer.putString(2,0)
    # r = "000102FF0000"
    # print(hexToBytes(r))
    # buffer.putInt(0x5a)
    # buffer.putInt(0x00)
    # buffer.putInt(0x01)
    # buffer.putString(4, "0011")
    # buffer.putString(4, 2)
    # buffer.putInt(0xff)
    # buffer.putShort(0)
    # buffer.putInt(0x88)
    # buffer.putInt(0x5a)
    # buffer.putInt(int("192"))
    # buffer.putBytes([12, 43, 54])
    print(buffer.hex)
    # tobytes = buffer.tobytes()
    # for b in tobytes:
    #     print(b)

    # dynamic_buffer = DynamicBuffer("0x" + buffer.hex)
    # print(dynamic_buffer)
    # print(dynamic_buffer.readBytes(32))
    # dynamic_buffer.setPos(8)
    # print(dynamic_buffer.pos)
    # print(dynamic_buffer.readBytes(32))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetGpiTrigger(Message):

    def __init__(self, gpiPort):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetGpiTrigger.value
        self.gpiPort = gpiPort
        self.triggerStart = None
        self.hexTriggerCommand = None
        self.triggerOver = None
        self.overDelayTime = None
        self.levelUploadSwitch = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.gpiPort)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.triggerStart = buffer.readInt()
            cmdLen = buffer.readShort()
            if cmdLen:
                self.hexTriggerCommand = bytesToHex(buffer.readBytes(cmdLen * 8))
            self.triggerOver = buffer.readInt()
            self.overDelayTime = buffer.readShort()
            self.levelUploadSwitch = buffer.readInt()
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.gpiPort, self.triggerStart, self.hexTriggerCommand, self.triggerOver, self.overDelayTime,
                    self.levelUploadSwitch))
        # return "{0},{1},{2},{3},{4},{5}".format(self.readerSerialNumber, self.powerOnTime, self.baseCompileTime,
        #                                     self.appVersions, self.systemVersions, self.appCompileTime)



# from time import *
from uhf.reader.protocol import *
from uhf.reader.communication import *
from uhf.reader.utils import *
# from json import *
#
# tidMap = {}
#
#
# def receiveEpc(epcInfo: LogBaseEpcInfo):
#     if epcInfo.result == 0:
#         print((epcInfo.epc, epcInfo.tid, epcInfo.userData, epcInfo.reserved))
#         if epcInfo.tid not in tidMap:
#             epcInfo.result = 1
#             tidMap[epcInfo.tid] = epcInfo
#         else:
#             tid_ = tidMap[epcInfo.tid]
#             tid_.result += 1
#             tidMap[epcInfo.tid] = tid_
#
#
# def callEpcOver():
#     print("---------------------")
#     sum = 0
#     for k, v in tidMap.items():
#         print(k, v.result)
#         sum += v.result
#     print(len(tidMap), "======", sum)
#
#
# def callTcpDisconnect(readName: ()):
#     print("断开连接", readName)
#     client.close()
#
#
# def MsgBaseStopTest():
#     stop = MsgBaseStop()
#     client.sendSynMsg(stop)
#     print(stop.rtMsg)
#
#
# def MsgAppGetReaderInfoTest():
#     # info = MsgAppGetReaderInfo()
#     # client.sendSynMsg(info)
#     # print(info)
#     client.sendSynBytes(hexToBytes("5A0001021000080000000101020006ED08"))
#
#
# def MsgAppSetSerialParamTest():
#     param = MsgAppSetSerialParam(2)
#     client.sendSynMsg(param)
#     print(param.rtMsg)
#
#
# def MsgAppGetSerialParamTest():
#     serialParam = MsgAppGetSerialParam()
#     print(client.sendSynMsg(serialParam))
#     print(serialParam)
#
#
# def MsgAppSetGpoTest():
#     gpo = MsgAppSetGpo(gpo1=1)
#     print(client.sendSynMsg(gpo))
#
#
# def MsgAppGetGpiStateTest():
#     state = MsgAppGetGpiState()
#     client.sendSynMsg(state)
#     print(state)
#
#
# def MsgAppSetEthernetIPTest():
#     ip = MsgAppSetEthernetIP(1, ip="192.168.1.168", mask="255.255.255.0", gateway="192.168.1.1")
#     print(client.sendSynMsg(ip))
#
#
# def MsgAppGetEthernetIPTest():
#     ip = MsgAppGetEthernetIP()
#     print(client.sendSynMsg(ip))
#     print(ip)
#
#
# def MsgAppGetReaderMacTest():
#     mac = MsgAppGetReaderMac()
#     print(client.sendSynMsg(mac))
#     print(mac)
#
#
# def MsgAppGetTcpModeTest():
#     tcpMode = MsgAppGetTcpMode()
#     print(client.sendSynMsg(tcpMode))
#     print(tcpMode)
#
#
# def MsgAppSetTcpModeTest():
#     setTcpMode = MsgAppSetTcpMode(0, serverPort=8160, clientIp="192.168.11.21", clientPort=8160)
#     print(client.sendSynMsg(setTcpMode))
#
#
# def MsgAppGetGpiTriggerTest():
#     getTrigger = MsgAppGetGpiTrigger(0)
#     print(client.sendSynMsg(getTrigger))
#     print(getTrigger)
#
#
# def MsgAppSetGpiTriggerTest():
#     setTrigger = MsgAppSetGpiTrigger(0, 2, "", 1, overDelayTime=10, levelUploadSwitch=1)
#     print(client.sendSynMsg(setTrigger))
#
#
# def MsgAppSetWiegandTest():
#     setWiegand = MsgAppSetWiegand(0, 0, 1)
#     print(client.sendSynMsg(setWiegand))
#     print(setWiegand)
#
#
# def MsgAppGetWiegandTest():
#     getWiegand = MsgAppGetWiegand()
#     print(client.sendSynMsg(getWiegand))
#     print(getWiegand)
#
#
# def MsgAppResetTest():
#     reset = MsgAppReset()
#     print(client.sendSynMsg(reset))
#     print(reset)
#
#
# def MsgAppSetReaderTimeTest():
#     setTime = MsgAppSetReaderTime(nowTimeSecond())
#     print(client.sendSynMsg(setTime))
#     print(setTime)
#
#
# def MsgAppGetReaderTimeTest():
#     getTime = MsgAppGetReaderTime()
#     print(client.sendSynMsg(getTime))
#     print(getTime.seconds)
#     print(secondFormat(getTime.seconds))
#
#
# def MsgAppRestoreDefaultTest():
#     restoreDefault = MsgAppRestoreDefault()
#     print(client.senUnSynMsg(restoreDefault))
#
#
# def MsgAppSetRs485Test():
#     setRs485 = MsgAppSetRs485(2)
#     print(client.sendSynMsg(setRs485))
#
#
# def MsgAppGetRs485Test():
#     getRs485 = MsgAppGetRs485()
#     print(client.sendSynMsg(getRs485))
#     print(getRs485)
#
#
# def MsgAppSetBreakpointResumeTest():
#     setBreakpointResume = MsgAppSetBreakpointResume(0)
#     print(client.sendSynMsg(setBreakpointResume))
#     print(setBreakpointResume)
#
#
# def MsgAppGetBreakpointResumeTest():
#     getBreakpointResume = MsgAppGetBreakpointResume()
#     print(client.sendSynMsg(getBreakpointResume))
#     print(getBreakpointResume)
#
#
# def MsgAppGetCacheTagDataTest():
#     getCacheTagData = MsgAppGetCacheTagData()
#     print(client.sendSynMsg(getCacheTagData))
#
#
# def MsgAppClearCacheDataTest():
#     clearCacheData = MsgAppClearCacheData()
#     print(client.sendSynMsg(clearCacheData))
#
#
# def MsgAppTagDataReplyTest():
#     tagDataReply = MsgAppTagDataReply(1)
#     print(client.sendSynMsg(tagDataReply))
#
#
# def MsgAppSetBeepOnOffTest():
#     setBeepOnOff = MsgAppSetBeepOnOff(0)
#     print(client.sendSynMsg(setBeepOnOff))
#
#
# def MsgAppSetBeepTest():
#     setBeep = MsgAppSetBeep(1, 0)
#     print(client.sendSynMsg(setBeep))
#
#
# def MsgAppGetWhiteListTest():
#     getWhiteList = MsgAppGetWhiteList(0)
#     print(client.sendSynMsg(getWhiteList))
#     print(getWhiteList)
#
#
# def MsgAppImportWhiteListTest():
#     importWhiteList = MsgAppImportWhiteList(0)
#     importWhiteList.packetContent = []
#     print(client.sendSynMsg(importWhiteList))
#
#
# def MsgAppDelWhiteListTest():
#     delWhiteList = MsgAppDelWhiteList()
#     print(client.sendSynMsg(delWhiteList))
#
#
# def MsgAppSetWhiteListActionTest():
#     setWhiteListAction = MsgAppSetWhiteListAction(1, 1)
#     print(client.sendSynMsg(setWhiteListAction))
#
#
# def MsgAppGetWhiteListActionTest():
#     getWhiteListAction = MsgAppGetWhiteListAction()
#     print(client.sendSynMsg(getWhiteListAction))
#     print(getWhiteListAction)
#
#
# def MsgAppGetWhiteListSwitchTest():
#     getWhiteListSwitch = MsgAppGetWhiteListSwitch()
#     print(client.sendSynMsg(getWhiteListSwitch))
#     print(getWhiteListSwitch)
#
#
# def MsgAppSetWhiteListSwitchTest():
#     setWhiteListSwitch = MsgAppSetWhiteListSwitch(0, 1)
#     print(client.sendSynMsg(setWhiteListSwitch))
#     print(setWhiteListSwitch)
#
#
# def MsgAppGetUdpParamTest():
#     getUdpParam = MsgAppGetUdpParam()
#     print(client.sendSynMsg(getUdpParam))
#     print(getUdpParam)
#
#
# def MsgAppSetUdpParamTest():
#     setUdpParam = MsgAppSetUdpParam(0, ip="192.168.1.100", port=8168, period=100)
#     print(client.sendSynMsg(setUdpParam))
#
#
# def MsgAppGetHttpParamTest():
#     getHttpParam = MsgAppGetHttpParam()
#     print(client.sendSynMsg(getHttpParam))
#     print(getHttpParam)
#
#
# def MsgAppSetHttpParamTest():
#     setHttpParam = MsgAppSetHttpParam(1, period=5, timeout=5, reportAddress="http://192.168.1.168:1090/Report")
#     print(client.sendSynMsg(setHttpParam))
#
#
# # 5A00010129002A 01 0005 00 0005 00 01 0020 687474703A2F2F3139322E3136382E312E3136383A383039302F5265706F7274DB22
#
# # 转化成字符串(序列化)
# # name_dumps = json.dumps(name)
# # 转化成json格式(反序列化)
# # name01_loads = json.loads(name01)
# # json.dumps 序列化时对中文默认使用的ascii编码.想输出真正的中文需要指定ensure_ascii=False：
# def MsgAppGetWifiHotspotSearchTest():
#     count = 0
#     while 1:
#         search = MsgAppGetWifiHotspotSearch(count)
#         if client.sendSynMsg(search)[0] == 0:
#             count += 1
#             if search.packetContent:
#                 # print(type(listToAscii(search.packetContent)))  # str
#                 result = loads(listToAscii(search.packetContent))  # type:dict
#                 print(result.items())
#             else:
#                 break
#
#
# def MsgAppSetWifiHotspotTest():
#     setWifiHotspot = MsgAppSetWifiHotspot("GX_5G", password="GX201909")
#     print(client.sendSynMsg(setWifiHotspot))
#
#
# def MsgAppGetWifiConnectStatusTest():
#     getWifiConnectStatus = MsgAppGetWifiConnectStatus()
#     print(client.sendSynMsg(getWifiConnectStatus))
#     print(getWifiConnectStatus)
#
#
# def MsgAppSetWifiIpTest():
#     setWifi = MsgAppSetWifiIp(1, ip="192.168.11.111", mask="255.255.255.0", gateway="192.168.11.1", dns1="192.168.11.1",
#                               dns2="0.0.0.0")
#     print(client.sendSynMsg(setWifi))
#
#
# def MsgAppGetWifiIpTest():
#     getWifi = MsgAppGetWifiIp(hotId=0)
#     print(client.sendSynMsg(getWifi))
#     print(getWifi)
#
#
# def MsgAppGetWifiSwitchTest():
#     getWifiSwitch = MsgAppGetWifiSwitch()
#     print(client.sendSynMsg(getWifiSwitch))
#     print(getWifiSwitch)
#
#
# def MsgAppSetWifiSwitchTest():
#     setWifiSwitch = MsgAppSetWifiSwitch(1)
#     print(client.sendSynMsg(setWifiSwitch))
#
#
# def MsgAppGetEasAlarmTest():
#     getEasAlarm = MsgAppGetEasAlarm()
#     print(client.sendSynMsg(getEasAlarm))
#     print(getEasAlarm)
#
#
# def MsgAppSetEasAlarmTest():
#     setEasAlarm = MsgAppSetEasAlarm(0, 0, 0, "1234", "FFFF", actionSuccess=ActionParamSuccess(1, gpo1=1),
#                                     actionFail=ActionParamFail(1, gpo2=0))
#     print(client.sendSynMsg(setEasAlarm))
#
#
# def MsgBaseGetCapabilitiesTest():
#     getCapabilities = MsgBaseGetCapabilities()
#     print(client.sendSynMsg(getCapabilities))
#     print(getCapabilities)
#
#
# def MsgBaseSetPowerTest():
#     dicPower = {"1": 20, "2": 20, "3": 21, "4": 23}
#     setPower = MsgBaseSetPower(**dicPower)
#     print(client.sendSynMsg(setPower))
#
#
# def MsgBaseGetPowerTest():
#     getPower = MsgBaseGetPower()
#     print(client.sendSynMsg(getPower))
#     print(getPower)
#
#
# def MsgBaseSetFreqRangeTest():
#     setFreqRange = MsgBaseSetFreqRange(3)
#     print(client.sendSynMsg(setFreqRange))
#
#
# def MsgBaseGetFreqRangeTest():
#     getFreqRange = MsgBaseGetFreqRange()
#     print(client.sendSynMsg(getFreqRange))
#     print(getFreqRange)
#
#
# def MsgBaseSetFrequencyTest():
#     fres = [0, 1, 2, 3, 4]
#     setFrequency = MsgBaseSetFrequency(0, *fres)
#     print(client.sendSynMsg(setFrequency))
#
#
# def MsgBaseGetFrequencyTest():
#     getFrequency = MsgBaseGetFrequency()
#     print(client.sendSynMsg(getFrequency))
#     print(getFrequency)
#
#
# def MsgBaseSetTagLogTest():
#     setTagLog = MsgBaseSetTagLog(repeatedTime=100)
#     print(client.sendSynMsg(setTagLog))
#
#
# def MsgBaseGetTagLogTest():
#     getTagLog = MsgBaseGetTagLog()
#     print(client.sendSynMsg(getTagLog))
#     print(getTagLog)
#
#
# def MsgBaseSetBasebandTest():
#     setBaseband = MsgBaseSetBaseband(baseSpeed=1, qValue=3, session=3, inventoryFlag=1)
#     print(client.sendSynMsg(setBaseband))
#
#
# def MsgBaseGetBasebandTest():
#     getBaseband = MsgBaseGetBaseband()
#     print(client.sendSynMsg(getBaseband))
#     print(getBaseband)
#
#
# def MsgBaseSetAutoDormancyTest():
#     setAutoDormancy = MsgBaseSetAutoDormancy(0, freeTime=10)
#     print(client.sendSynMsg(setAutoDormancy))
#
#
# def MsgBaseGetAutoDormancyTest():
#     getAutoDormancy = MsgBaseGetAutoDormancy()
#     print(client.sendSynMsg(getAutoDormancy))
#     print(getAutoDormancy)
#
#
# def MsgBaseInventoryEpcTest():
#     # epc_filter = ParamEpcFilter(0, 32, "1234")
#     inventoryEpc = MsgBaseInventoryEpc(1, 1, readTid=ParamEpcReadTid(0, 6), readUserData=ParamEpcReadUserData(0, 2),
#                                        readReserved=ParamEpcReadReserved(0, 2), hexPassword=None)
#     print(client.sendSynMsg(inventoryEpc))
#
#
# def MsgBaseWriteEpcTest():
#     temp = "2222"
#     hexData = getPcValue(temp) + temp
#     writeEpc = MsgBaseWriteEpc(1, 0, 2, hexData, hexPassword=None)
#     print(client.sendSynMsg(writeEpc))
#
#
# def MsgBaseLockEpcTest():
#     lockEpc = MsgBaseLockEpc(1, 1, 0, hexPassword="08002222")
#     print(client.sendSynMsg(lockEpc))
#
#
# def MsgBaseInventory6bEpcTest():
#     inventory6b = MsgBaseInventory6b(1, 1, 0, readUserData=Param6bReadUserData(0, 2))
#     print(client.sendSynMsg(inventory6b))
#
#
# def received6b(bInfo: LogBase6bInfo):
#     if bInfo.result == 0:
#         print(bInfo.tid)
#
#
# def call6bOver(bOver: LogBase6bOver):
#     print(bOver.rtMsg)
#
#
# def MsgBaseWrite6bTest():
#     write6b = MsgBaseWrite6b(1, "E004000030BCE808", 8, "ffff")
#     print(client.sendSynMsg(write6b))
#
#
# def MsgBaseLock6bGetTest():
#     lock6bGet = MsgBaseLock6bGet(1, "E004000030BCE808", 8)
#     print(client.sendSynMsg(lock6bGet))
#     print(lock6bGet.lockState)
#
#
# def MsgBaseInventoryGbTest():
#     inventoryGb = MsgBaseInventoryGb(1, 1)
#     print(client.sendSynMsg(inventoryGb))
#
#
# def receivedGb(gbInfo: LogBaseGbInfo):
#     if gbInfo.result == 0:
#         print(gbInfo.epc)
#
#
# def callGbOver(gbOver: LogBaseGbOver):
#     print(gbOver.rtMsg)
#
#
# def MsgBaseWriteGbTest():
#     temp = "2222"
#     hexData = getGbPcValue(temp) + temp
#     writeEpc = MsgBaseWriteGb(1, 0x10, 1, hexData, hexPassword=None)
#     print(client.sendSynMsg(writeEpc))
#
#
# if __name__ == '__main__':
#     try:
#         client = GClient()
#         client.callEpcInfo = receiveEpc
#         client.callEpcOver = callEpcOver
#         client.call6bInfo = received6b
#         client.call6bOver = call6bOver
#         client.callGbInfo = receivedGb
#         client.callGbOver = callGbOver
#         client.callDisconnected = callTcpDisconnect
#         stop = MsgBaseStop()
#         # if client.openTcp(("192.168.1.168", 8160)):
#         # if client.openSerial485(("COM7", 115200, 1), timeout=0):
#         # print(localtime())
#         # print(datetime.now())
#         # print(localtime())
#         # print(int(mktime(strptime(strftime("%Y-%m-%d %H:%M:%S", localtime()), "%Y-%m-%d %H:%M:%S"))))
#         if client.openSerial(("COM7", 115200), timeout=0):
#             print(strftime("%Y-%m-%d %H:%M:%S", localtime()))
#             print("========================输入指定数字执行操作========================")
#             events = {0: MsgBaseStopTest, 1: MsgAppGetReaderInfoTest, 2: MsgAppSetSerialParamTest,
#                       3: MsgAppGetSerialParamTest, 4: MsgAppSetGpoTest, 5: MsgAppGetGpiStateTest,
#                       6: MsgAppSetEthernetIPTest, 7: MsgAppGetEthernetIPTest, 8: MsgAppGetReaderMacTest,
#                       9: MsgAppSetTcpModeTest, 10: MsgAppGetTcpModeTest, 11: MsgAppSetGpiTriggerTest,
#                       12: MsgAppGetGpiTriggerTest, 13: MsgAppSetWiegandTest, 14: MsgAppGetWiegandTest,
#                       15: MsgAppResetTest, 16: MsgAppSetReaderTimeTest, 17: MsgAppGetReaderTimeTest,
#                       18: MsgAppRestoreDefaultTest, 19: MsgAppSetRs485Test, 20: MsgAppGetRs485Test,
#                       21: MsgAppSetBreakpointResumeTest, 22: MsgAppGetBreakpointResumeTest,
#                       23: MsgAppGetCacheTagDataTest, 24: MsgAppClearCacheDataTest, 25: MsgAppTagDataReplyTest,
#                       26: MsgAppSetBeepOnOffTest, 27: MsgAppSetBeepTest, 28: MsgAppGetWhiteListTest,
#                       29: MsgAppDelWhiteListTest, 30: MsgAppSetWhiteListActionTest, 31: MsgAppGetWhiteListActionTest,
#                       32: MsgAppGetWhiteListSwitchTest, 33: MsgAppSetWhiteListSwitchTest, 34: MsgAppSetUdpParamTest,
#                       35: MsgAppGetUdpParamTest, 36: MsgAppGetHttpParamTest, 37: MsgAppSetHttpParamTest,
#                       38: MsgAppGetWifiHotspotSearchTest, 39: MsgAppSetWifiHotspotTest,
#                       40: MsgAppGetWifiConnectStatusTest, 41: MsgAppSetWifiIpTest, 42: MsgAppGetWifiIpTest,
#                       43: MsgAppGetWifiSwitchTest, 44: MsgAppSetWifiSwitchTest, 45: MsgAppGetEasAlarmTest,
#                       46: MsgAppSetEasAlarmTest, 47: MsgBaseGetCapabilitiesTest, 48: MsgBaseSetPowerTest,
#                       49: MsgBaseGetPowerTest, 50: MsgBaseSetFreqRangeTest, 51: MsgBaseGetFreqRangeTest,
#                       52: MsgBaseSetFrequencyTest, 53: MsgBaseGetFrequencyTest, 54: MsgBaseSetTagLogTest,
#                       55: MsgBaseGetTagLogTest, 56: MsgBaseSetBasebandTest, 57: MsgBaseGetBasebandTest,
#                       58: MsgBaseSetAutoDormancyTest, 59: MsgBaseGetAutoDormancyTest, 60: MsgBaseInventoryEpcTest,
#                       61: MsgBaseWriteEpcTest, 62: MsgBaseLockEpcTest, 63: MsgBaseInventory6bEpcTest,
#                       64: MsgBaseWrite6bTest, 65: MsgBaseLock6bGetTest, 66: MsgBaseInventoryGbTest}
#             for key, value in events.items():
#                 print(key, "-", value.__name__, end="\t\t")
#                 if key % 5 == 0 and key != 0:
#                     print()
#             while True:
#                 s = input()
#                 if s.isdigit():
#                     if int(s) in events:
#                         events[int(s)].__call__()
#
#             # print(strftime("%Y-%m-%d %H:%M:%S", localtime()))
#
#
#
#     except Exception as ex:
#         raise ex
#         # print(ex.args)
#
#         # raise ex



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseGetCapabilities(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_GetCapabilities.value
        self.minPower = None
        self.maxPower = None
        self.antennaCount = None
        self.frequencyArray = []
        self.protocolArray = []

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.minPower = buffer.readInt()
            self.maxPower = buffer.readInt()
            self.antennaCount = buffer.readInt()
            freLen = buffer.readShort()
            if freLen:
                self.frequencyArray = [buffer.readInt() for i in range(freLen)]
            proLen = buffer.readShort()
            if proLen:
                self.protocolArray = [buffer.readInt() for i in range(proLen)]
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.minPower, self.maxPower, self.antennaCount, self.frequencyArray, self.protocolArray))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppTagDataReply(Message):

    def __init__(self, serialNumber: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_TagDataReply.value
        self.serialNumber = serialNumber

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.serialNumber)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.serialNumber = buffer.readLong()
            self.rtCode = 0



# from uhf.reader.communication import CommunicationInter
# import threading
# from hid import *
#
#
# class HidClient(CommunicationInter):
#
#     def __init__(self):
#         super().__init__()
#         self.client = device()
#         self.lock = threading.Condition()
#
#     def open(self, param, **kwargs):
#         try:
#             self.client.open_path(param)
#             self.client.set_nonblocking(True)
#             self.keepReceived = True
#             received = threading.Thread(target=self.startReceived, name="HidReceived")
#             received.start()
#             processRing = threading.Thread(target=self.startProcess, name="RingBuffer")
#             processRing.start()
#             return True
#         except Exception as ex:
#             print(ex.args)
#
#     def sendMsg(self, message):
#         try:
#             message.pack()
#             arr = message.toByte(False)  # type:bytes
#             self.sendBytes(arr)
#         except Exception as ex:
#             print(ex.args)
#             # raise ex
#
#     def sendBytes(self, bytes):
#         tempList = list(bytes)
#         tempLen = len(tempList)
#         dataLen = tempLen
#         for i in range(divmod(tempLen, 64)[0] + 1):
#             if dataLen > 64:
#                 dataLen -= 64
#                 self.client.write([0] + tempList[i * 64:(i + 1) * 64])
#             else:
#                 len_ = tempList[i * 64:(i * 64) + dataLen]
#                 repair = [0 for i in range(64 - dataLen)]
#                 len_.extend(repair)
#                 self.client.write([0] + len_)
#
#     def close(self):
#         self.keepReceived = False
#         self.client.close()
#         with self.ringLock:
#             self.ringLock.notifyAll()
#
#     def startReceived(self):
#         while self.keepReceived:
#             try:
#                 with self.ringLock:
#                     recv = self.client.read(64, 1000)
#                     if recv:
#                         self.ringBuffer.writeData(recv)
#                         self.ringLock.notify()
#                         self.ringLock.wait()
#
#             except Exception as ex:
#                 # pass
#                 print(ex.args)
#                 self.keepReceived = False
#                 # raise ex
#         else:
#             pass



def listToAscii(arr: list):
    if arr:
        return "".join(list(map(chr, arr)))


def bytesToAscii(arr):
    if arr:
        b = bytearray()
        b.extend(arr)
        return b.decode(encoding='ascii')



def hexToBytes(hex):
    """
    hex转bytearray
    :param hex:
    :return:
    """
    return bytearray.fromhex(hex)


def bytesToHex(arr):
    """
    字节数组转hex
    :param arr: 字节数组
    :return:hex
    """
    # if isinstance(arr, (list, bytearray, bytes)):
    try:
        return bytearray(arr).hex()
    except Exception as ex:
        print(ex.args)
        return None


def hexToInt(hexValue):
    """
    hex转int
    :param hexValue:hex
    :return: int
    """
    return int.from_bytes(hexToBytes(hexValue), byteorder='big', signed=False)


def listToAscii(list):
    """
    字节数组转ascii
    :param list: 字节数组
    :return: ascii
    """
    return "".join([chr(x) for x in list])



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestR2000ReadWrite(Message):

    def __init__(self, operationType: int, registerAddress: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_R2000ReadWrite.value
        self.operationType = operationType
        self.registerAddress = registerAddress
        self.writeContent = kwargs.get("writeContent", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.operationType)
        buffer.putShort(self.registerAddress)
        if self.writeContent:
            buffer.putInt(0x01)
            buffer.putShort(self.writeContent)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success.", 1: "Failed."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)
            if len(self.cData) > 1:
                dataBuffer = DynamicBuffer("0x" + bytesToHex(self.cData))
                dataBuffer.pos = 8
                if dataBuffer.readInt() == 1:
                    self.writeContent = dataBuffer.readShort()



from uhf.reader.protocol.message import Message
from uhf.reader.protocol.enumg import EnumG
from uhf.reader.utils.byteBuffer import DynamicBuffer
from uhf.reader.utils.HexUtils import *


class LogBaseEpcInfo(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseLogMid_Epc.value
        self.readerName = None
        self.epc = None
        self.bEpc = None
        self.pc = None
        self.antId = None
        self.rssi = None
        self.result = 0
        self.tid = None
        self.bTid = None
        self.userData = None
        self.bUser = None
        self.reserved = None
        self.bRes = None
        self.childAntId = None
        self.strUtc = None
        self.frequencyPoint = None
        self.phase = None
        self.epcData = None
        self.bEpcData = None
        self.ctesiusLtu27 = None
        self.ctesiusLtu31 = None
        self.readerSerialNumber = None
        self.replySerialNumber = None
        self.kunYue = None
        self.rssidBm = None
        self.cykeoRule = None

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x"+hex)
            epcLength = buffer.readShort()
            self.bEpc = buffer.readBytes(epcLength * 8)
            if self.bEpc:
                self.epc = bytesToHex(self.bEpc)
            self.pc = buffer.readShort()
            self.antId = buffer.readInt()
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    self.rssi = buffer.readInt()
                elif pid == 2:
                    self.result = buffer.readInt()
                elif pid == 3:
                    tidLen = buffer.readShort()
                    self.bTid = buffer.readBytes(tidLen * 8)
                    if self.bTid:
                        self.tid = bytesToHex(self.bTid)
                elif pid == 4:
                    userLen = buffer.readShort()
                    self.bUser = buffer.readBytes(userLen * 8)
                    if self.bUser:
                        self.userData = bytesToHex(self.bUser)
                elif pid == 5:
                    resLen = buffer.readShort()
                    self.bRes = buffer.readBytes(resLen * 8)
                    if self.bRes:
                        self.reserved = bytesToHex(self.bRes)
                elif pid == 6:
                    self.childAntId = buffer.readInt()
                elif pid == 7:
                    utcSecond = buffer.readLong() * 1000
                    utcMicrosecond = buffer.readLong() / 1000
                    self.strUtc = utcSecond + utcMicrosecond
                    # print(ms)
                elif pid == 8:
                    self.frequencyPoint = buffer.readLong()
                elif pid == 9:
                    self.phase = buffer.readInt()
                elif pid == 10:
                    epcDataLen = buffer.readShort()
                    self.bEpcData = buffer.readBytes(epcDataLen * 8)
                    if self.bEpcData:
                        self.epcData = bytesToHex(self.bEpcData)
                elif pid == 0x11:
                    self.ctesiusLtu27 = buffer.readShort()
                elif pid == 0x12:
                    self.ctesiusLtu31 = buffer.readShort()
                elif pid == 0x13:
                    self.kunYue = buffer.readShort()
                elif pid == 0x14:
                    self.rssidBm = buffer.readShort()
                elif pid == 0x20:
                    snLen = buffer.readShort()
                    if snLen:
                        self.readerSerialNumber = "".join([chr(x) for x in buffer.readBytes(snLen * 8)])
                elif pid == 0x22:
                    self.replySerialNumber = buffer.readLong()
                elif pid == 0x24:
                    self.cykeoRule = buffer.readInt()

                    




from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetHttpParam(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetHttpParam.value
        self.onOrOff = None
        self.period = None
        self.formats = None
        self.timeout = None
        self.openCache = None
        self.reportAddress = None  # type:str

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.onOrOff = buffer.readInt()
            self.period = buffer.readShort()
            self.formats = buffer.readInt()
            self.timeout = buffer.readShort()
            self.openCache = buffer.readInt()
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    reLen = buffer.readShort()
                    if reLen:
                        self.reportAddress = listToAscii(buffer.readBytes(reLen * 8))
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.onOrOff, self.period, self.timeout, self.reportAddress))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetReaderMac(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetReaderMac.value
        self.mac = None

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hexData = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hexData)
            self.mac = hex(buffer.readInt())[2:].zfill(2) + "-" + hex(buffer.readInt())[2:].zfill(2) + "-" + hex(
                buffer.readInt())[2:].zfill(2) + "-" + hex(
                buffer.readInt())[2:].zfill(2) + "-" + hex(buffer.readInt())[2:].zfill(2) + "-" + hex(buffer.readInt())[
                                                                                                  2:].zfill(2)
            self.rtCode = 0

    def __str__(self) -> str:
        return self.mac



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseSetResidenceTime(Message):

    def __init__(self, onOff: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_SetResidenceTime.value
        self.antLingeringTime = kwargs.get("antLingeringTime", None)
        self.freLingeringTime = kwargs.get("freLingeringTime", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.antLingeringTime is not None:
            buffer.putInt(0x01)
            buffer.putShort(self.antLingeringTime)
        if self.freLingeringTime is not None:
            buffer.putInt(0x02)
            buffer.putShort(self.freLingeringTime)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Parameter error.",
                      2: "Other error.", 3: "Save failure."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetGpo(Message):

    def __init__(self, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetGpo.value
        self.gpo1 = kwargs.get("gpo1", None)
        self.gpo2 = kwargs.get("gpo2", None)
        self.gpo3 = kwargs.get("gpo3", None)
        self.gpo4 = kwargs.get("gpo4", None)
        self.gpo5 = kwargs.get("gpo5", None)
        self.gpo6 = kwargs.get("gpo6", None)
        self.gpo7 = kwargs.get("gpo7", None)
        self.gpo8 = kwargs.get("gpo8", None)
        self.gpo9 = kwargs.get("gpo9", None)
        self.gpo10 = kwargs.get("gpo10", None)
        self.gpo11 = kwargs.get("gpo11", None)
        self.gpo12 = kwargs.get("gpo12", None)
        self.gpo13 = kwargs.get("gpo13", None)
        self.gpo14 = kwargs.get("gpo14", None)
        self.gpo15 = kwargs.get("gpo15", None)
        self.gpo16 = kwargs.get("gpo16", None)
        self.gpo17 = kwargs.get("gpo17", None)
        self.gpo18 = kwargs.get("gpo18", None)
        self.gpo19 = kwargs.get("gpo19", None)
        self.gpo20 = kwargs.get("gpo20", None)
        self.gpo21 = kwargs.get("gpo21", None)
        self.gpo22 = kwargs.get("gpo22", None)
        self.gpo23 = kwargs.get("gpo23", None)
        self.gpo24 = kwargs.get("gpo24", None)
        self.gpo25 = kwargs.get("gpo25", None)
        self.gpo26 = kwargs.get("gpo26", None)
        self.gpo27 = kwargs.get("gpo27", None)
        self.gpo28 = kwargs.get("gpo28", None)
        self.gpo29 = kwargs.get("gpo29", None)
        self.gpo30 = kwargs.get("gpo30", None)
        self.gpo31 = kwargs.get("gpo31", None)
        self.gpo32 = kwargs.get("gpo32", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.gpo1 is not None:
            buffer.putInt(0x01)
            buffer.putInt(self.gpo1)
        if self.gpo2 is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.gpo2)
        if self.gpo3 is not None:
            buffer.putInt(0x03)
            buffer.putInt(self.gpo3)
        if self.gpo4 is not None:
            buffer.putInt(0x04)
            buffer.putInt(self.gpo4)
        if self.gpo5 is not None:
            buffer.putInt(0x05)
            buffer.putInt(self.gpo5)
        if self.gpo6 is not None:
            buffer.putInt(0x06)
            buffer.putInt(self.gpo6)
        if self.gpo7 is not None:
            buffer.putInt(0x07)
            buffer.putInt(self.gpo7)
        if self.gpo8 is not None:
            buffer.putInt(0x08)
            buffer.putInt(self.gpo8)
        if self.gpo9 is not None:
            buffer.putInt(0x09)
            buffer.putInt(self.gpo9)
        if self.gpo10 is not None:
            buffer.putInt(0x0A)
            buffer.putInt(self.gpo10)
        if self.gpo11 is not None:
            buffer.putInt(0x0B)
            buffer.putInt(self.gpo11)
        if self.gpo12 is not None:
            buffer.putInt(0x0C)
            buffer.putInt(self.gpo12)
        if self.gpo13 is not None:
            buffer.putInt(0x0D)
            buffer.putInt(self.gpo13)
        if self.gpo14 is not None:
            buffer.putInt(0x0E)
            buffer.putInt(self.gpo14)
        if self.gpo15 is not None:
            buffer.putInt(0x0F)
            buffer.putInt(self.gpo15)
        if self.gpo16 is not None:
            buffer.putInt(0x10)
            buffer.putInt(self.gpo16)
        if self.gpo17 is not None:
            buffer.putInt(0x11)
            buffer.putInt(self.gpo17)
        if self.gpo18 is not None:
            buffer.putInt(0x12)
            buffer.putInt(self.gpo18)
        if self.gpo19 is not None:
            buffer.putInt(0x13)
            buffer.putInt(self.gpo19)
        if self.gpo20 is not None:
            buffer.putInt(0x14)
            buffer.putInt(self.gpo20)
        if self.gpo21 is not None:
            buffer.putInt(0x15)
            buffer.putInt(self.gpo21)
        if self.gpo22 is not None:
            buffer.putInt(0x16)
            buffer.putInt(self.gpo22)
        if self.gpo23 is not None:
            buffer.putInt(0x17)
            buffer.putInt(self.gpo23)
        if self.gpo24 is not None:
            buffer.putInt(0x18)
            buffer.putInt(self.gpo24)
        if self.gpo25 is not None:
            buffer.putInt(0x19)
            buffer.putInt(self.gpo25)
        if self.gpo26 is not None:
            buffer.putInt(0x1A)
            buffer.putInt(self.gpo26)
        if self.gpo27 is not None:
            buffer.putInt(0x1B)
            buffer.putInt(self.gpo27)
        if self.gpo28 is not None:
            buffer.putInt(0x1C)
            buffer.putInt(self.gpo28)
        if self.gpo29 is not None:
            buffer.putInt(0x1D)
            buffer.putInt(self.gpo29)
        if self.gpo30 is not None:
            buffer.putInt(0x1E)
            buffer.putInt(self.gpo30)
        if self.gpo31 is not None:
            buffer.putInt(0x1F)
            buffer.putInt(self.gpo31)
        if self.gpo32 is not None:
            buffer.putInt(0x20)
            buffer.putInt(self.gpo32)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Port parameter reader hardware is not supported ."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestWorkModeGet(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_ReaderWorkModeGet.value
        self.workMode = None
        self.rs485BaudRate = None
        self.rs485DataBit = None
        self.rs485ParityBit = None
        self.rs485StopBit = None
        self.rs232BaudRate = None

    def bytesToClass(self):
        pass

    def pack(self):
        pass

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.workMode = buffer.readInt()
            self.rs485BaudRate = buffer.readInt()
            self.rs485DataBit = buffer.readInt()
            self.rs485ParityBit = buffer.readInt()
            self.rs485StopBit = buffer.readInt()
            self.rs232BaudRate = buffer.readInt()
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.workMode, self.rs485BaudRate, self.rs485DataBit, self.rs485ParityBit, self.rs485StopBit,
                    self.rs232BaudRate))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestCarrierWave(Message):

    def __init__(self, antennaNum: int, frequencyNum: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_CarrierWave.value
        self.antennaNum = antennaNum
        self.frequencyNum = frequencyNum

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaNum)
        buffer.putInt(self.frequencyNum)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Frequency parameter reader is not supported.",
                      2: "Port parameter reader is not supported.", 3: "Lock failure.", 4: "Other error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseGetResidenceTime(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_GetResidenceTime.value
        self.antLingeringTime = None
        self.freLingeringTime = None

    def bytesToClass(self):
        pass

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    self.antLingeringTime = buffer.readShort()
                if pid == 2:
                    self.freLingeringTime = buffer.readShort()

            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.antLingeringTime, self.freLingeringTime))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestEnvRssiAutoDetection(Message):

    def __init__(self, antennaEnable: int, startFrequency: int, endFrequency: int, mode: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_EnvRssiAutoDetection.value
        self.antennaEnable = antennaEnable
        self.startFrequency = startFrequency
        self.endFrequency = endFrequency
        self.mode = mode

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putLong(self.antennaEnable)
        buffer.putShort(self.startFrequency)
        buffer.putShort(self.endFrequency)
        buffer.putInt(self.mode)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "FrequencyPoint Param Reader Not Support.", 2: "Port Param Reader Not Support.",
                      3: "Phase-locked loop locking failed", 4: "Other error"}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



import socket
import threading
from uhf.reader.communication.tcp_client import TcpClient


class TcpServer(object):

    def __init__(self):
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.keepListen = False
        self.listenPort = 8160
        self.serverSocket = None
        self.onRemoteConnected = None

    def open(self, port: int, address="0.0.0.0"):
        if self.serverSocket:
            return False
        self.keepListen = True
        self.listenPort = port
        self.server.bind((address, self.listenPort))
        self.server.listen(9999)  # 设置最大连接数，超过后排队
        listen = threading.Thread(target=self.startListen, name="TcpServerListen")
        listen.start()
        return True

    def startListen(self):
        while self.keepListen:
            try:
                # 建立客户端连接
                self.serverSocket, addr = self.server.accept()
                # print("连接地址: %s" % str(addr))
                if self.onRemoteConnected:
                    client = TcpClient()
                    client.addr = addr
                    # self.serverSocket.setblocking(False)
                    if client.openSocket(self.serverSocket):
                        self.onRemoteConnected(client)
            except Exception as ex:
                print("stop listen")
                # print(ex.args)

    def close(self):
        try:
            self.keepListen = False
            self.server.close()
        except Exception as ex:
            print("stop listen")



# from uhf.reader.communication import CommunicationInter
# from uhf.reader.protocol import *
# from uhf.reader.utils import *
# import threading
# import usb.core
# import usb.util
# import platform
#
#
# class UsbHidClient(CommunicationInter):
#
#     def __init__(self):
#         super().__init__()
#         # self.dev = usb.core.find(idVendor=0x03EB, idProduct=0x2421)
#         self.__inPoint = None
#         self.__outPoint = None
#         self.dev = None  # type:usb.core.Device
#         self.lock = threading.Condition()
#
#     def open(self, dev: usb.core.Device, **kwargs):
#         try:
#             if dev:
#                 self.dev = dev
#                 if platform.system().lower() == 'linux':
#                     if self.dev.is_kernel_driver_active(0):
#                         self.dev.detach_kernel_driver(0)
#
#                 self.dev.set_configuration()
#                 configuration = self.dev.get_active_configuration()
#                 cfg_ = configuration[(0, 0)]
#                 self.__outPoint = usb.util.find_descriptor(
#                     cfg_,
#                     custom_match=lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_OUT)
#                 self.__inPoint = usb.util.find_descriptor(
#                     cfg_,
#                     custom_match=lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_IN)
#                 if self.__outPoint and self.__inPoint is None:
#                     return False
#                 self.keepReceived = True
#                 received = threading.Thread(target=self.startReceived, name="HidReceived",
#                                             args=(self.dev,))
#                 received.start()
#                 processRing = threading.Thread(target=self.startProcess, name="RingBuffer")
#                 processRing.start()
#                 return True
#             else:
#                 return False
#         except Exception as ex:
#             print(ex.args)
#
#     def sendMsg(self, message: Message):
#         try:
#             message.pack()
#             arr = message.toByte(False)  # type:bytes
#             self.sendBytes(arr)
#         except Exception as ex:
#             print(ex.args)
#             # raise ex
#
#     # 传输大字节待解决
#     def sendBytes(self, bytes):
#         tempList = list(bytes)
#         tempLen = len(tempList)
#         dataLen = tempLen
#         for i in range(divmod(tempLen, 64)[0] + 1):
#             if dataLen > 64:
#                 dataLen -= 64
#                 self.__outPoint.write(tempList[i * 64:(i + 1) * 64])
#             else:
#                 len_ = tempList[i * 64:(i * 64) + dataLen]
#                 repair = [0 for i in range(64 - dataLen)]
#                 len_.extend(repair)
#                 self.__outPoint.write(len_)
#
#     def close(self):
#         # usb.util.release_interface(self.dev, self.__inPoint)
#         # usb.util.release_interface(self.dev, self.__outPoint)
#         self.keepReceived = False
#         with self.ringLock:
#             self.ringLock.notifyAll()
#             self.ringLock = None
#
#     def startReceived(self, dev):
#         while self.keepReceived:
#             try:
#                 with self.ringLock:
#                     recv = self.__inPoint.read(64)
#                     # recv = self.dev.read(self.__inPoint, 64)
#                     if recv:
#                         self.ringBuffer.writeData(recv)
#                         self.ringLock.notify()
#                         self.ringLock.wait()
#             except Exception as ex:
#                 pass
#                 # print(ex.args)
#                 # raise ex
#         else:
#             usb.util.dispose_resources(dev)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class ParamSafeEncipheredData(Parameter):

    def __init__(self, start: int, enData: str):
        self.start = start
        self.enData = enData

    def bytesToClass(self, dataBytes):
        hex = bytesToHex(dataBytes)
        buffer = DynamicBuffer("0x" + hex)
        self.start = buffer.readShort()
        dataLen = buffer.readShort()
        self.enData = bytesToHex(buffer.readBytes(dataLen * 8))
        return self

    def toBytes(self):
        buffer = DynamicBuffer()
        buffer.putShort(self.start)
        buffer.putBytes(hexToBytes(self.enData))
        return buffer.tobytes()

    def __str__(self) -> str:
        return str((self.start, self.enData))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetTcpMode(Message):

    def __init__(self, tcpMode, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetTcpMode.value
        self.tcpMode = tcpMode
        self.serverPort = kwargs.get("serverPort", None)
        self.clientIp = kwargs.get("clientIp", None)
        self.clientPort = kwargs.get("clientPort", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.tcpMode)
        if self.tcpMode == 0:  # 服务器模式
            if self.serverPort is not None:
                buffer.putInt(0x01)
                buffer.putShort(self.serverPort)
        else:  # 客户端模式
            if self.clientIp is not None:
                buffer.putInt(0x02)
                splitIp = self.clientIp.split('.')
                for ips in splitIp:
                    buffer.putInt(int(ips))
            if self.clientPort is not None:
                buffer.putInt(0x03)
                buffer.putShort(self.clientPort)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Server IP parameter error ."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



import struct


class DataStream(bytearray):

    def append(self, v, fmt='>B'):
        self.extend(struct.pack(fmt, v))


if __name__ == '__main__':
    x = DataStream()
    x.append(5)
    print(x)
    x.append(0xff, '>B')
    print(x)
    x.append(0x12,">H")
    print(x)
    print(x.hex())

# Character	Byte order	Size	Alignment
# @(默认)	本机	本机	本机,凑够4字节
# =	本机	标准	none,按原字节数
# <	小端	标准	none,按原字节数
# >	大端	标准	none,按原字节数
# !	network(大端)	标准	none,按原字节数
#
# 格式符	C语言类型	Python类型	Standard size
# x	pad byte(填充字节)	no value	 
# c	char	string of length 1	1
# b	signed char	integer	1
# B	unsigned char	integer	1
# ?	_Bool	bool	1
# h	short	integer	2
# H	unsigned short	integer	2
# i	int	integer	4
# I(大写的i)	unsigned int	integer	4
# l(小写的L)	long	integer	4
# L	unsigned long	long	4
# q	long long	long	8
# Q	unsigned long long	long	8
# f	float	float	4
# d	double	float	8
# s	char[]	string	 
# p	char[]	string	 
# P	void *	long	 
# 注- -!

# _Bool在C99中定义,如果没有这个类型,则将这个类型视为char,一个字节;
# q和Q只适用于64位机器;
# 每个格式前可以有一个数字,表示这个类型的个数,如s格式表示一定长度的字符串,4s表示长度为4的字符串;4i表示四个int;
# P用来转换一个指针,其长度和计算机相关;
# f和d的长度和计算机相关;



from uhf.reader.utils.byteBuffer import DynamicBuffer
from uhf.reader.utils.HexUtils import *


class RingBuffer(object):

    def __init__(self):
        super().__init__()
        self.buffer = DynamicBuffer()
        self.buffer.pos = 0
        self.dataCount = 0
        self.dataHead = 0
        self.dataEnd = 0

    def reset(self):
        # self.buffer.pos = 0
        self.dataCount = 0
        self.dataHead = 0
        self.dataEnd = 0

    def writeData(self, data: bytes):
        self.buffer.putBytes(data)
        self.dataEnd += len(data) * 8
        self.dataCount += len(data) * 8

    def readData(self, bitLen):
        if self.dataCount:
            if bitLen <= self.dataCount:
                read_bitLen_bytes = self.buffer.readBytes(bitLen)
                self.dataHead += bitLen
                self.dataCount -= bitLen
                return read_bitLen_bytes

    def readBit(self, bitLen):
        if self.dataCount:
            if bitLen <= self.dataCount:
                self.dataHead += bitLen
                self.dataCount -= bitLen
                bit_len = self.buffer.readBitLen(bitLen)
                return bit_len
            else:
                return 0

    def indexData(self, pos):
        if pos < self.dataCount and self.dataCount > 0:
            read_int = self.buffer.readInt()
            self.buffer.pos -= 8
            return read_int

    def cleanData(self, bitLen):
        if bitLen > self.dataCount:
            self.reset()
            self.buffer.clear()
        else:
            self.dataHead += bitLen
            self.dataCount -= bitLen
            self.buffer.pos += bitLen

    def cleanAll(self):
        if self.dataHead == self.dataEnd:
            self.buffer.clear()

    def subPos(self, pos):
        if pos <= self.buffer.pos:
            self.buffer.pos -= pos
            self.dataCount += pos
            self.dataHead -= pos


if __name__ == '__main__':
    buffer = RingBuffer()
    buffer.writeData(hexToBytes("AAbbccddeeff"))
    print(hex(buffer.indexData(0)))
    print(buffer)
    print(bytesToHex(buffer.readData(32)))
    print(buffer)
    print(bytesToHex(buffer.readData(32)))
    print(buffer)
    buffer.writeData(hexToBytes("112211221122"))
    print(buffer.readBit(8))
    print(bytesToHex(buffer.readData(32)))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetHttpParam(Message):

    def __init__(self, onOrOff: int, period: int, timeout: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetHttpParam.value
        self.onOrOff = onOrOff
        self.period = period
        self.formats = kwargs.get("formats", 0)
        self.timeout = timeout
        self.openCache = kwargs.get("openCache", 0)
        self.reportAddress = kwargs.get("reportAddress", None)  # type:str

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.onOrOff)
        buffer.putShort(self.period)
        buffer.putInt(self.formats)
        buffer.putShort(self.timeout)
        buffer.putInt(self.openCache)
        if self.reportAddress is not None:
            buffer.putInt(0x01)
            buffer.putShort(len(self.reportAddress))
            buffer.putBytes(self.reportAddress.encode())
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Fail"}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)




from uhf.reader.protocol.message import Message
from uhf.reader.protocol.enumg import EnumG
from uhf.reader.utils.byteBuffer import DynamicBuffer
from uhf.reader.utils.HexUtils import *


class LogBaseGbInfo(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseLogMid_Gb.value
        self.readerName = None
        self.epc = None
        self.bEpc = None
        self.tid = None
        self.bTid = None
        self.pc = None
        self.antId = None
        self.rssi = None
        self.result = 0
        self.userData = None
        self.bUser = None
        self.readerSerialNumber = None

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            epcLength = buffer.readShort()
            self.bEpc = buffer.readBytes(epcLength * 8)
            if self.bEpc:
                self.epc = bytesToHex(self.bEpc)
            self.pc = buffer.readShort()
            self.antId = buffer.readInt()
            while buffer.pos / 8 < len(self.cData):
                pid = buffer.readInt()
                if pid == 1:
                    self.rssi = buffer.readInt()
                elif pid == 2:
                    self.result = buffer.readInt()
                elif pid == 3:
                    tidLen = buffer.readShort()
                    self.bTid = buffer.readBytes(tidLen * 8)
                    if self.bTid:
                        self.tid = bytesToHex(self.bTid)
                elif pid == 4:
                    userLen = buffer.readShort()
                    self.bUser = buffer.readBytes(userLen * 8)
                    if self.bUser:
                        self.userData = bytesToHex(self.bUser)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppSetWifiIp(Message):

    def __init__(self, autoIp: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_SetWifiIp.value
        self.autoIp = autoIp
        self.ip = kwargs.get("ip", None)
        self.mask = kwargs.get("mask", None)
        self.gateway = kwargs.get("gateway", None)
        self.dns1 = kwargs.get("dns1", None)
        self.dns2 = kwargs.get("dns2", None)
        self.hotId = kwargs.get("hotId", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.autoIp)
        if self.autoIp == 1:
            if self.ip is not None:
                buffer.putInt(0x01)
                splitIp = self.ip.split('.')
                for ips in splitIp:
                    buffer.putInt(int(ips))

            if self.mask is not None:
                buffer.putInt(0x02)
                masks = self.mask.split('.')
                for ms in masks:
                    buffer.putInt(int(ms))

            if self.gateway is not None:
                buffer.putInt(0x03)
                gateways = self.gateway.split('.')
                for gs in gateways:
                    buffer.putInt(int(gs))

            if self.dns1 is not None:
                buffer.putInt(0x04)
                dns1s = self.dns1.split('.')
                for d1 in dns1s:
                    buffer.putInt(int(d1))

            if self.dns2 is not None:
                buffer.putInt(0x05)
                dns2s = self.dns2.split('.')
                for d2 in dns2s:
                    buffer.putInt(int(d2))
        if self.hotId is not None:
            buffer.putInt(0x06)
            buffer.putLong(self.hotId)
        else:
            buffer.putInt(0x06)
            buffer.putLong(0)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "ReaderIp parameter error ."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



import threading
from uhf.reader.utils import *
from uhf.reader.protocol import *


class CommunicationInter(object):

    def __init__(self):
        self.serialNumber = None
        self.rs485Address = 0
        self.isRs485 = False
        self.isSendHeartbeat = False
        self.keepReceived = False
        self.ringBuffer = RingBuffer()
        self.ringLock = threading.Condition()
        self.onMessageReceived = None
        self.onTcpDisConnect = None
        # self.callBackEpcInfo = None
        # self.callBackEpcOver = None
        # self.callBack6bInfo = None
        # self.callBack6bOver = None
        # self.callBackGbInfo = None
        # self.callBackGbOver = None

    def open(self, param, **kwargs):
        pass

    def openSocket(self, serverSocket):
        pass

    def sendMsg(self, message):
        # message.pack()
        pass

    def sendBytes(self, bytes):
        pass

    def close(self):
        pass

    def callMessage(self, msg: Message):
        if self.onMessageReceived:
            self.onMessageReceived(msg)

    def callDisConnect(self, addr):
        if self.onTcpDisConnect:
            self.onTcpDisConnect(addr)

    def startProcess(self):
        while self.keepReceived:
            try:
                # print('thread %s is running...' % threading.current_thread().name)
                with self.ringLock:
                    if self.ringBuffer.dataCount < 48:
                        self.ringLock.notify()
                        self.ringLock.wait()
                    if self.ringBuffer.indexData(0) != EnumG.Msg_Header.value:
                        self.ringBuffer.cleanData(1 * 8)
                        continue
                    else:
                        temp = DynamicBuffer()
                        if self.ringBuffer.dataCount >= 56:
                            if self.isRs485:
                                temp.putBytes(self.ringBuffer.readData(48))
                            else:
                                temp.putBytes(self.ringBuffer.readData(40))
                            dataLen = self.ringBuffer.readBit(16)
                            if dataLen != 0:
                                temp.putShort(dataLen)
                            if (dataLen + 2) * 8 > self.ringBuffer.dataCount:
                                if dataLen > 1024:
                                    self.ringBuffer.cleanData(1 * 8)
                                else:
                                    self.ringBuffer.subPos(temp.len)
                                    self.ringLock.notify()
                                    self.ringLock.wait()
                                continue
                            else:
                                temp.putBytes(self.ringBuffer.readData(dataLen * 8))
                                data = self.ringBuffer.readData(16)
                                temp.putBytes(data)
                                msg = Message().new(temp.hex)
                                if msg:
                                    if msg.checkCrc():
                                        if self.isRs485 and self.rs485Address != msg.rs485Address:
                                            continue
                                        self.ringBuffer.cleanAll()
                                        self.callMessage(msg)
                                    else:
                                        print("crc错误")
                                else:
                                    print("-------------------------------解析错误--------------------------------")
                                    self.ringBuffer.cleanData(temp.len)
                        else:
                            self.ringLock.notify()
                            self.ringLock.wait()
            except Exception as ex:
                # pass
                print(ex.args)
                raise ex



from uhf.reader.protocol import *
from uhf.reader.utils import *


class ParamEpcReadTid(Parameter):

    def __init__(self, mode: int, dataLen: int):
        self.area = mode
        self.dataLen = dataLen

    def bytesToClass(self, dataBytes):
        hex = bytesToHex(dataBytes)
        buffer = DynamicBuffer("0x" + hex)
        self.area = buffer.readInt()
        self.dataLen = buffer.readInt()

        return self

    def toBytes(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.area)
        buffer.putInt(self.dataLen)
        return buffer.tobytes()

    def __str__(self) -> str:
        return str((self.area, self.dataLen))



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgAppGetWifiIp(Message):

    def __init__(self, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_App.value
        self.msgId = EnumG.AppMid_GetWifiIp.value
        self.hotId = kwargs.get("hotId", 0)
        self.autoIp = None
        self.ip = None
        self.mask = None
        self.gateway = None
        self.dns1 = None
        self.dns2 = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.hotId is not None:
            buffer.putInt(0x01)
            buffer.putLong(self.hotId)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.autoIp = buffer.readInt()
            self.ip = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt())
            self.mask = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt())
            self.gateway = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt()) + "." + str(
                buffer.readInt())
            self.dns1 = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt()) + "." + str(
                buffer.readInt())
            self.dns2 = str(buffer.readInt()) + "." + str(buffer.readInt()) + "." + str(
                buffer.readInt()) + "." + str(
                buffer.readInt())
            self.rtCode = 0

    def __str__(self) -> str:
        return str((self.autoIp, self.ip, self.mask, self.gateway, self.dns1, self.dns2))



from uhf.reader.protocol.message import Message
from uhf.reader.protocol.enumg import EnumG


class LogBaseGJbOver(Message):

    def __init__(self):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseLogMid_GJb.value
        self.readerSerialNumber = None
        self.readerName = None

    def pack(self):
        super().pack()

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Single operation complete.", 1: "Receive stop instruction.",
                      2: "A hardware failure causes an interrupt."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from .interface import CommunicationInter
from .tcp_server import TcpServer
from .tcp_client import TcpClient
from .serial_client import SerialClient
# from .usb_hid import UsbHidClient
from .gserver import GServer
from .gclient import GClient
# from .hid import HidClient
# "UsbHidClient",
"HidClient"
__all__ = ["GClient", "GServer", "TcpClient", "TcpServer", "SerialClient",  "CommunicationInter",
           ]



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestPowerCalibrationGet(Message):

    def __init__(self, childFreqRange: int, power: int):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_PowerCalibrationGet.value
        self.childFreqRange = childFreqRange
        self.power = power
        self.powerParam = None

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.childFreqRange)
        buffer.putInt(self.power)
        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            hex = bytesToHex(self.cData)
            buffer = DynamicBuffer("0x" + hex)
            self.childFreqRange = buffer.readInt()
            self.power = buffer.readInt()
            self.powerParam = buffer.readInt()
            self.rtCode = 0

    def __str__(self) -> str:
        return str(self.powerParam)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgTestWorkModeSet(Message):

    def __init__(self, workMode: int, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Test.value
        self.msgId = EnumG.TestMid_ReaderWorkModeSet.value
        self.workMode = workMode
        self.rs485BaudRate = kwargs.get("rs485BaudRate", None)
        self.rs485DataBit = kwargs.get("rs485DataBit", None)
        self.rs485ParityBit = kwargs.get("rs485ParityBit", None)
        self.rs485StopBit = kwargs.get("rs485StopBit", None)
        self.rs232BaudRate = kwargs.get("rs232BaudRate", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        buffer.putInt(self.workMode)
        if self.rs485BaudRate and self.rs485DataBit and self.rs485ParityBit and self.rs485StopBit:
            buffer.putInt(0x01)
            buffer.putInt(self.rs485BaudRate)
            buffer.putInt(self.rs485DataBit)
            buffer.putInt(self.rs485ParityBit)
            buffer.putInt(self.rs485StopBit)
        if self.rs232BaudRate:
            buffer.putInt(0x02)
            buffer.putInt(self.rs232BaudRate)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success.", 1: "Other error."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)



from uhf.reader.protocol import *
from uhf.reader.utils import *


class MsgBaseSafeAttestation(Message):

    def __init__(self, **kwargs):
        super().__init__()
        self.mt_8_11 = EnumG.Msg_Type_Bit_Base.value
        self.msgId = EnumG.BaseMid_SafeAttestation.value
        self.token1 = kwargs.get("token1", None)
        self.token2Result = kwargs.get("token2Result", None)
        self.encipheredData = kwargs.get("encipheredData", None)  # type:ParamSafeEncipheredData
        self.key = kwargs.get("key", None)

    def bytesToClass(self):
        pass

    def pack(self):
        buffer = DynamicBuffer()
        if self.token1 is not None:
            buffer.putInt(0x01)
            buffer.putBytes(hexToBytes(self.token1))
        if self.token2Result is not None:
            buffer.putInt(0x02)
            buffer.putInt(self.token2Result)
        if self.encipheredData is not None:
            buffer.putInt(0x03)
            data_to_bytes = self.encipheredData.toBytes()
            buffer.putShort(len(data_to_bytes))
            buffer.putBytes(data_to_bytes)
        if self.key is not None:
            buffer.putInt(0x04)
            to_bytes = hexToBytes(self.key)
            buffer.putShort(len(to_bytes))
            buffer.putBytes(to_bytes)

        self.cData = buffer.tobytes()
        self.dataLen = buffer.len / 8

    def unPack(self):
        if self.cData:
            dirMsg = {0: "Success", 1: "Frequency parameter reader is not supported.",
                      2: "Save failure."}
            self.rtCode = self.cData[0]
            if self.rtCode in dirMsg:
                self.rtMsg = dirMsg.get(self.rtCode, None)
